package cologne.eck.all_peas.control;


/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Parent class of all pea frames except image pea. 
 */

import java.awt.Window;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.cloud.gui.UploadWorker;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.NewPasswordDialog;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;

/**
 * Content related utilities. 
 * 
 * @author Axel von dem Bruch
 *
 */
public abstract class ContentUtilities {

	/**
	 * Encrypt and store the current content of a file 
	 * whether as ciphertext or as plaintext with a key. 
	 * The key is then stored encrypted as session key. 
	 * 
	 * @param keyMaterial	the key to encrypt: must be null if cipherBytes are not null
	 * @param plainBytes	the content to encrypt and to store if cipherBytes is null
	 * @param cipherBytes	the encrypted content to store if plainBytes is null
	 * @param fileName		the file to store the encrypted content with absolute path
	 * 
	 * @return	true if content was saved successfully
	 */
	public final static boolean saveContent(
			byte[] keyMaterial, byte[] plainBytes, byte[] cipherBytes, 
			String fileName, boolean encryptBySessionKey) {

		if (new File(fileName).exists() == false) {
			TestLog.e(ContentUtilities.class, "ContentUtilities save: file does not exist: " + fileName);
			new UnexpectedValueException("String", "fileName", "does not eist: " + fileName).printDescription();
			return false;
		}
		if (new File(fileName).isFile() == false) {
			TestLog.e(ContentUtilities.class, "ContentUtilities save: not file " + fileName);
			new UnexpectedValueException("String", "fileName", "is not a name of a file: " + fileName).printDescription();
			return false;
		}
		if (plainBytes != null && cipherBytes != null) {
			TestLog.e(ContentUtilities.class, "ContentUtilities save: One parameter too much " + fileName);
			new UnexpectedValueException("byte[]", "cipherBytes and plainBytes", "both not null for file: " + fileName).printDescription();
			return false;
		}
		if (plainBytes == null && cipherBytes == null) {
			TestLog.e(ContentUtilities.class, "ContentUtilities save: Required parameter is null: " + fileName);
			new UnexpectedValueException("byte[]", "cipherBytes and plainBytes", "both null for file: " + fileName).printDescription();
			return false;
		}
		if (cipherBytes != null && keyMaterial != null) {
			TestLog.e(ContentUtilities.class, "ContentUtilities save: One parameter too much " + fileName);
			new UnexpectedValueException("byte[]", "cipherBytes and keyMaterial", "both not null for file: " + fileName).printDescription();
			return false;
		}

		if (cipherBytes == null) {
			cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, 
				keyMaterial, // keyMaterial is overwritten
				encryptBySessionKey, null ); // key is stored encrypted
		}
		if (cipherBytes == null) {
			TestLog.e(ContentUtilities.class, "Decryption error");
			TestLog.ve(ContentUtilities.class, "Missing ciphertext", 5);
			return false;
		}

		boolean success = WriteResources.write(cipherBytes,fileName, null);
		if (success == false) {
			new UnexpectedValueException("boolean", "success", 
					"is false for writing: " + fileName).printDescription();
		}
		
		if (fileName.contains(CloudControl.getTmpDir())) {
			// save in cloud:
			saveTmpFileInCloud(fileName);
		}
		return success;
	}
	/**
	 * Save one single backup file of a cloud file
	 * that is stored in temporary directory
	 * 
	 * @param tmpFileName	the name of the backup file in tmp folder to store
	 */
	private static void saveTmpFileInCloud(String tmpFileName) {
		if (tmpFileName.contains(CloudControl.getTmpDir())) {
			// save in cloud:
			String providerName = CloudControl.getProviderNameFromTmpFile(tmpFileName);
			File[] fileArray = {new File(tmpFileName)};

			new UploadWorker(JREProperties.getMainWindow(), 
					providerName, 
					null, 
					fileArray, 
					null).execute();
		} else {
			TestLog.e(ContentUtilities.class, "ContentUtilities saveTmpFileInCloud: " + tmpFileName
					+ " - is not a cloud file backup");
		}
	}
	/**
	 * Get a new password from a password dialog, derive a new key, 
	 * encrypt and save the content of files with the new key. 
	 * The new key will then be stored encrypted as session key and 
	 * will replace a previously stored key. 
	 * A wait cursor is set inside this function during the process.
	 * This method should be called in EDT.
	 * (currently only used by LockFrameImage)
	 * 
	 * @param fileNames		the names of other files to change the password, 
	 * 						the content of the files must be encrypted
	 * 						with the current session key
	 * @param window		the owner for the NewPasswordDialog
	 */
	public static final void changePassword(String[] fileNames, Object window){

		if (fileNames == null) {
			TestLog.e(ContentUtilities.class, "No file names: Can't change password...");
			new UnexpectedValueException("fileNames", "String[]", "is null").printDescription();
			return;
		}
		Window win = null;
		try{
			win = (Window) window;
		} catch (Exception e){
			TestLog.e(ContentUtilities.class, e.toString() + " - " + e.getLocalizedMessage(), 5);
			new UnexpectedValueException("window", "Window", "is not correct type").printDescription();
		}
		// get the currently used key to decrypt several encrypted files
		// unfortunately the key must be present for a while... 
		byte[] oldKeyMaterial = null;
		if (fileNames != null) {
			oldKeyMaterial = CipherStuff.getSessionKeyCrypt().getKey();
			if (PeaProperties.isTestMode()){
				Comparator.checkNullVector(oldKeyMaterial);
			}
		}
		// TODO test
		// change the cloud passwords of all opened cloud providers: 
		// decrypt with old key, later: encrypt with new key
		Set<String> openedProviders = ((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel().getCloudControl().getUsedProvidersAsSet();
		// store provider names and plain passwords
		HashMap<String, String> providersAndPlainPasswords = new HashMap<String, String>();
		if (openedProviders != null && openedProviders.size() > 0) {
			for (String providerName : openedProviders) {
				String plainPsw = CloudPropertyHandler.getPlainPasswordFromProvider(providerName, 
						Converter.bytes2hex(KeyDerivation.getSalt()), 
						Converter.bytes2hex(PeaProperties.getAlgoParamString()));
				if (plainPsw != null) {
					providersAndPlainPasswords.put(providerName, plainPsw);
				} else {
					TestLog.ve(ContentUtilities.class, "Missing password in properties for " + providerName);
				}
			}
		}
		
		// get the new password:
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(window, 
				PeaProperties.getVmBridge().translate("change_password"), null,
				PswDialogView.getView().getLocation());
		char[] newPsw = newPswDialog.getDialogInput();	
		byte[] newKeyMaterial = null;
		try {
			// set the wait cursor: 
			if (win != null) {
				win.setCursor(CursorManager.getWaitCursor());
			}

			// derive the new key from the new password, using the same salt
			newKeyMaterial = PeaControl.deriveKeyFromPsw(newPsw);
			if (newKeyMaterial == null) {
				if (newPsw != null) {
					Zeroizer.zero(newPsw);
				}
				return;
			}
			if (PeaProperties.isTestMode()){
				Comparator.checkNullVector(newKeyMaterial);
			}

			// clear the new password
			if (newPsw != null){
				Zeroizer.zero(newPsw);
			}

			// change the password for the other file names:
			for (int i = 0; i < fileNames.length; i++) {	
				
				// read the encrypted content of the files 
				byte[] otherCipherBytes = ReadResources.readExternFile(fileNames[i]); 
				if (otherCipherBytes == null) {
					TestLog.e(ContentUtilities.class, "ContentUtilities, password change: "
							+ "Reading content of encrypted file failed: " 
							+ fileNames[i]);
					PeaDialog.showMessage(window, 							
							PeaProperties.getVmBridge().translate("cant_read_content_of_file") + ": \n"
							+ fileNames[i], 
							PeaProperties.getVmBridge().translate("error"), 
							0);//JOptionPane.ERROR_MESSAGE
					// reset the wait cursor (modal dialog may occurred): 
					if (win != null) {
						win.setCursor(CursorManager.getWaitCursor());
					}
					continue;
				}
				if (PeaProperties.isTestMode()){
					Comparator.checkNullVector(oldKeyMaterial);
				}
				// decrypt the encrypted content in RAM
				byte[] otherPlainBytes = CipherStuff.getInstance().decrypt(
						otherCipherBytes, 
						Arrays.copyOf(oldKeyMaterial, oldKeyMaterial.length), // use the old key to decrypt
						false );	// clear the key, do not encrypt by session key
									// (otherwise this would overwrite the new key)
				if (otherPlainBytes == null) {
					TestLog.e(ContentUtilities.class, "ContentUtilities, password change: "
							+ "Decrypting content of encrypted file failed: " 
							+ fileNames[i]);
					PeaDialog.showMessage(window, PeaProperties.getVmBridge().translate("decryption_failed") + ": \n"
							+ fileNames[i], 
							PeaProperties.getVmBridge().translate("error"), 
							0);//JOptionPane.ERROR_MESSAGE
					// reset the wait cursor (modal dialog may occurred): 
					if (win != null) {
						win.setCursor(CursorManager.getWaitCursor());
					}					
					continue;
				}
				if (PeaProperties.isTestMode()){
					Comparator.checkNullVector(newKeyMaterial);
				}
				// encrypt the plain text content with the new key:
				// (can't use saveContent, because keyMaterial must not be stored)
				byte[] newKeyCipherBytes = CipherStuff.getInstance().encrypt(otherPlainBytes, 
						Arrays.copyOf(newKeyMaterial, newKeyMaterial.length), // keyMaterial is overwritten
						false, null ); // do not encrypt by session key: clear key
				if (newKeyCipherBytes == null) {
					TestLog.e(ContentUtilities.class, "ContentUtilities, password change: "
							+ "Encrypting content of file failed: " 
							+ fileNames[i]);
					PeaDialog.showMessage(window, 
							PeaProperties.getVmBridge().translate("encryption_failed")+ " : \n"
							+ fileNames[i], 
							PeaProperties.getVmBridge().translate("error"), 
							0);//JOptionPane.ERROR_MESSAGE
					// reset the wait cursor (modal dialog may occurred): 
					if (win != null) {
						win.setCursor(CursorManager.getWaitCursor());
					}					
					continue;
				}

				boolean success = WriteResources.write(newKeyCipherBytes, fileNames[i], null);
				if (success == false) {
					TestLog.e(ContentUtilities.class, "ContentUtilities, password change: "
							+ "Writing content of file failed: " 
							+ fileNames[i]);
					PeaDialog.showMessage(window, 
							PeaProperties.getVmBridge().translate("cant_write_content_of_file") + ": \n"
							+ fileNames[i], 
							PeaProperties.getVmBridge().translate("error"), 
							0);//JOptionPane.ERROR_MESSAGE
					// reset the wait cursor (modal dialog may occurred): 
					if (win != null) {
						win.setCursor(CursorManager.getWaitCursor());
					}					
					continue;
				}
			} // for file name iteration
			
		} finally {
			// reset the cursor:
			if (win != null) {
				win.setCursor(CursorManager.getDefaultCursor());
			}
		}
		// zeroize the old key
		Zeroizer.zero(oldKeyMaterial);
		// encrypt and store the new key
		if (PeaProperties.isTestMode()){
			Comparator.checkNullVector(newKeyMaterial);
		}

		CipherStuff.getSessionKeyCrypt().storeKey(newKeyMaterial);
		// zeroize the key used here
		Zeroizer.zero(newKeyMaterial);
		
		// change the cloud provider passwords in cloud.properties
		// for all successfully opened providers: 
		String saltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
		String algoParam = Converter.bytes2hex(PeaProperties.getAlgoParamString());
		if (providersAndPlainPasswords.isEmpty() == false) {
			for(HashMap.Entry<String, String> entrySet: providersAndPlainPasswords.entrySet()){
				String providerName = entrySet.getKey();
				String psw = entrySet.getValue();
				String hexEncryptedPsw = CloudPropertyHandler.encryptPassword(psw);
				String error = CloudPropertyHandler.storeEncryptedPasswordFromProvider(hexEncryptedPsw, providerName, 
						saltAsHex, algoParam, null);
				//String error = CloudPropertyHandler.setSingleLoginProperty(providerName + "_password", hexEncryptedPsw, null);
				if (error != null) {
					TestLog.e(ContentUtilities.class, "Resetting cloud provider password failed for " + providerName);
				}
				TestLog.v(ContentUtilities.class, "Cloud provider password in properties changes: " + providerName);
				//Zeroizer.zero(psw);
			}
		}
	}
}
