package cologne.eck.all_peas.control;





/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * This is the base class for all Password Encrypted Archives (peas).
 * This is the parent class of all PswDialog* classes. 
 */


//import java.awt.Window;
import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;

import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.FilePanelHolder;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREBridge;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;

import settings.*;



public abstract class PeaControl { 

	
	// for Image Lock PEA and Notebook PEA: the chosen file names to decrypt
	private static ArrayList<String> availableFileNames = new ArrayList<String>();	
	
	private static PeaControl dialog;
	
	private static char[] initializedPassword = null;
	
	// the panel to display encrypted files
//	private static FileTypePanel encryptedFileTypePanel;
	
	// the name of the property file for a specific PEA
	private static String peaPropFileName = null;
	
	// the current main window: lockFrame or PswDialogView
	private static boolean plainTextMode = false;//Window mainWindow = null;
	
	private static String derivationError = null;
	
	//============================================================
	// --- abstract Methods:
	//


	// pre computation while windowOpening
	// this is nowhere used yet, but maybe used later 
	/**
	 * Used for pre-computations while the user types the password
	 */
	public abstract void preComputeInThread();
	//
	// before exit: clear secret values
	/**
	 * Overwriting secret value before closing the program
	 */
	public abstract void clearSecretValues();
	/**
	 * Start the decryption.
	 * This method should be called in EDT.
	 */
	public abstract void startDecryption();	
	
	// used in menu of PswDialogView:
	/**
	 * Display a file chooser to initialize (encrypt)
	 * new files. This method should be called in EDT.
	 */
	public abstract void initializeNewFiles();		
	
	// used in menu of PswDialogView:
	/**
	 * Display a file chooser to open
	 * previously encrypted files. This method should be called in EDT.
	 */
	public abstract void openEncryptedFiles();	
	
	// currently only used in PeaProperties.getFileType():
	/**
	 * Get the chosen file names to decrypt
	 * 
	 * @return	an array of Strings of the selected file names
	 */
	public abstract String[] getSelectedFileNames();
	
	/**
	 * Get the name of the used PEA as String
	 * in one word e.g. File-Lock-Pea
	 * 
	 * @return the name of the PEA in one word
	 */
	public abstract String getPeaName();
	
	/**
	 * Get the version of the used PEA as String
	 * 
	 * @return the version as String
	 */
	public abstract String getVersion();
	
	/**
	 * Get the year of the publication of the used PEA
	 * 
	 * @return	year of publication as String
	 */
	public abstract String getYearOfPublication();
	
	/**
	 * Get the specific website of the used PEA, that is the 
	 * web address following the language, for example "/html/file_pea.html"
	 * 
	 * @return	the website as String
	 */
	public abstract String getWebsite();
	
	/**
	 * Get the link with the source code of the used PEA
	 * 
	 * @return	the link as String
	 */
	public abstract String getSourceLink();
	
	/**
	 * Get the frame with the decrypted content
	 * 
	 * @return	the frame of the opened PEA
	 */
	public abstract Object getLockFrame();
	
	
	/**
	 * Get the supported file extensions for this PEA
	 * without leading . in lower case to open encrypted files
	 * (e.g. ics for calendar, rtf for notes,
	 * png, jpg, jepg, bmp, gif for image). If this method returns null, 
	 * all extensions are to be opened
	 * 
	 * @return	the supported file extensions for this PEA without leading . 
	 * 			or null (open all types of files - FileLock PEA)
	 */
	public abstract String[] getSupportedExtensions();
	
	/**
	 * Check if the given file is in the expected format 
	 * for the specific PEA: For Notebook PEA: txt or rtf,
	 * for Image Lock PEA: png, jpg, bmp,gif, 
	 * for File Lock PEA: nothing to check.
	 * 
	 * @param file				the file to check
	 * @param extensionCheck		check only the file name extension,
	 * 							this is also possible for encrypted files
	 * @param middleCheck
	 * @param expensiveCheck
	 * @return
	 */
	public abstract boolean checkFormat(File file, boolean extensionCheck, boolean middleCheck, boolean expensiveCheck);

	// only used in Notebook PEA:
	/**
	 * Get currently used plain text
	 * 
	 * @return	the current plain text
	 */
	protected abstract byte[] getPlainBytes();

	/**
	 * Set all properties for this specific PEA from property file
	 * @param prop	properties for this specific PEA
	 * @param inputStream 	the InputStream to read the properties
	 * 
	 * @return	errorMessage or null
	 */
	public abstract String setPeaPropValues(Properties prop, InputStream inputStream);
	
	/**
	 * Update all properties for this specific PEA from current instance
	 * 
	 * @param prop	properties for this specific PEA
	 * @param outputStream 	the OutputStream to write the properties
	 * 
	 * @return	errorMessage or null
	 */
	public abstract String updatePeaPropValues(Properties prop);//, OutputStream outputStream);
	
	//============================================================

	/**
	 * Start the execution of the PEA: stop the collection of entropy, 
	 * add file to path file, start decryption process
	 * 
	 * @param rememberPath		remember the selected file names or not
	 */
	public final void startExecution(boolean rememberPath) {
		
		PswDialogView.setStarted(true);

		EntropyPool.getInstance().stopCollection();

		if (PswDialogView.isInitializing() == false) {
			
			if ( ( ! PeaProperties.getFileType().equals("file"))) {
				// cleanup cloud properties: delete empty sub sets:
				CloudPropertyHandler.cleanupProperties();
			}
			
			if (PeaProperties.isCommandLineMode() == false) {

				FileTypePanel ftp = (FileTypePanel)JREProperties.getTypePanel();
				// set Model:
				if (ftp == null || (! PeaProperties.getFileType().equals("file") && ftp.getCloudFilePanel() == null) ) {
					TestLog.e(PeaControl.class, "Unexpected error when accessing cloud files");
					TestLog.ve(PeaControl.class, "Missing CloudFilePanel", 5);
					return;
	//				this.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().setCloudFileModel(ftp.getCloudFilePanel().getCloudFileModel());
				}
				if ( ftp.getFileComposer().getFileModel().checkForValidFile() == false){
					// check for cloud file
					if ( ( ! PeaProperties.getFileType().equals("file")) // there is no cloud file panel in File Lock PEA
							&& ftp.getCloudFilePanel().getComponentCount() > 0) {
						// get and decrypt the cloud files in XControl	
						//TestLog.o(getClass(), "cloud file found");			
					} else {
						PswDialogView.setMessage(PeaProperties.getVmBridge().translate("no_valid_file_selected"), true, false);
						PswDialogView.enableOkButton();
						return;
					}
				}
				// remember this file?
				if (rememberPath == true ) {
					String[] newNames = PeaControl.getDialog().getSelectedFileNames();
					if (newNames != null) {
						PathFileManager.addFileNamesToPathFile( newNames );
					} else {
						TestLog.e(PeaControl.class, "PeaControl.startExecution: files to remember are null");
					}
				}
				//TestLog.v(PeaControl.class, "PeaControl.startExecution",5);
				PeaControl.getDialog().startDecryption();
			} else {
				PeaControl.getDialog().startDecryption();
			}


		} else { //  initializing

			Attachments.setFileIdentifier(AttachmentHandler.getVersion2ID());
			PeaProperties.setAlgoParamString(
					PeaProperties.getDefaultAlgoParamString().getBytes(AttachmentHandler.getASCIICharset()));

/*			if (PeaProperties.getLastSalt() != null) {
				KeyDerivation.setSalt(PeaProperties.getLastSalt());
			} else {
				TestLog.v(PeaControl.class, "no lastSalt property");
			}*/
			PeaControl.getDialog().startDecryption();
	// TODO for all PEAs??? Always ask to remember? Or only check box?
			// but all files from FileTypePanel
			// remember this file?
	/*		if (PeaProperties.isInsideJar() == false) {

				if (rememberPath == true ) {
					
					String[] newName = { PeaControl.getAvailableFileNames().get(0) };
					PathFileManager.addFilesToPathFile( newName);
				}
			} */
		} 
		PswDialogView.setStarted(false);
	}
	
	/**
	 * Set salt and ask to take previous salt:
	 * 1. from other files in FileTypePanel
	 * 2. from lastSalt property
	 * This method should be called in EDT.
	 */
/*	public void setSaltForNewFiles() {
		String[] shownFileNames = ((FileTypePanel)JREProperties.getTypePanel()).getValidFileNames(false, false);
	//	String[] shownFileNames = dialog.getEncryptedFileTypePanel().getValidFileNames(false, false);
		if (shownFileNames != null && shownFileNames.length > 0){
			//if (PeaProperties.getLastSalt() != null)
			// ask to take salt from files
			//String questionMessage = "Do you want to open the new files together with the previous files?";
			int result = PeaDialog.showQuestion(
					//FileControl.dialogView, 
					PswDialogView.getView(),
			//		PeaProperties.getOsBridge().translate("add_new_files_when_decrypted") + "\n" +
							 PeaProperties.getVmBridge().translate("open_files_together"), 
							PeaProperties.getVmBridge().translate("init_new_file"),
							0, 1);
			if (result == 0){ // yes  ok
				boolean saltSuccess = setSaltFromEncryptedFiles(shownFileNames);
				if (saltSuccess == false) {
					// reset salt if password failed before:
					// set new salt to attach to file 
					KeyDerivation.setSalt(
							new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
					PeaDialog.showMessage(
							PswDialogView.getView(),
							PeaProperties.getVmBridge().translate("operation_failed")
							+ "\n" + PeaProperties.getVmBridge().translate("open_files_separately"));
					TestLog.e(PeaControl.class, "New files can not be opened together with previous files");
				}
			} else {
				// reset salt if password failed before:
				// set new salt to attach to file and xor with programRandomBytes
				KeyDerivation.setSalt(
						new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
			}
		} else {
			// ask to take salt from properties:
			byte[] lastSalt = PeaProperties.getLastSalt();
			if (lastSalt == null) {

				// reset salt if password failed before:
				// set new salt to attach to file and xor with programRandomBytes
				KeyDerivation.setSalt(
						new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
			} else {
				// ask
				int result = PeaDialog.showQuestion(
						PswDialogView.getView(),
						PeaProperties.getVmBridge().translate("open_files_together"), 
						PeaProperties.getVmBridge().translate("init_new_file"),
						0, 1);
				if (result == 0){ // yes  ok
					TestLog.o(getClass(), "Set salt from lastSalt property...");
					KeyDerivation.setSalt(lastSalt);
				} else {
					KeyDerivation.setSalt(
							new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
				}
			}
		}
	} */
	
	/**
	 * Set the salt from previous files from FileTypePanel.
	 * Takes the attached salt from the first file and sets
	 * this salt for the next key derivation process
	 * 
	 * @param fileNames	 file names from panel
	 * 
	 * @return		true: successfully set the salt
	 * 				false: salt setting failed
	 */
/*	public boolean setSaltFromEncryptedFiles(String[] fileNames){
		if (fileNames == null || fileNames.length == 0){
			TestLog.e(PeaControl.class, "No file to set salt from...");
			return false;
		}
		String saltContainingFile = null;
		// look for the first file with salt (no directory)
		for (int i = 0; i < fileNames.length; i++) {					
			if (new File(fileNames[i]).isFile() ){
				saltContainingFile = fileNames[i];
				break;
			} else {
				continue;
			}						
		}
		if (saltContainingFile == null) { // no file found
			TestLog.e(PeaControl.class, "missing salt containing file");
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("no_valid_file_selected") + " (salt missing)", true, false);
			return false;
		} else {
			if (PeaProperties.getWorkingMode().equals("-t")){
				TestLog.o(getClass(), "Get salt from file " + saltContainingFile );
			}				
		}

		// get and set the attached salt:
		int saltLen = KeyDerivation.getSaltSize();
		byte[] end = null;
		try {
			end = Attachments.getEndBytesOfFile( 
					saltContainingFile,//chosenFileNames[0], 
					Attachments.getFileIdentifierSize() 
					+ saltLen);			
			byte[] attachedSalt = new byte[saltLen];
			System.arraycopy(end,  0,  attachedSalt,  0,  saltLen);
			// reset salt if password failed before:
			KeyDerivation.setSalt(attachedSalt);
		} catch (IOException e) {
			TestLog.e(PeaControl.class, new ExceptionHelper(e).getInfos());
			TestLog.e(PeaControl.class, 
						"###" + // translate here, not in PswDialogView
								PeaProperties.getVmBridge().translate("file_inappropriate")
								+ saltContainingFile//chosenFileNames[0] 
								+ PeaProperties.getVmBridge().translate("caused_error")
								+ e.toString() + ", " + e.getMessage() + "\n"
								+ PeaProperties.getVmBridge().translate("attached_salt_not_found"));	
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("file_inappropriate"), true, false);
			return false;
		} catch (Exception e) {
			TestLog.e(PeaControl.class, new ExceptionHelper(e).getInfos());
			PswDialogView.setMessage("unexpected_error", true, false);
			if (PeaProperties.getWorkingMode().equals("-t")){
				TestLog.e(PeaControl.class, new ExceptionHelper(e).getInfos());
				e.printStackTrace();
			}
			return false;
		}
	
		return true;
	} */
	/**
	 * Check if the attached salt of an encrypted file equals
	 * the previously set salt
	 * 
	 * @param fileName	the file to check
	 * 
	 * @return	true, if the salt of the file fits the existing salt, 
	 * 			false if salt differs or an error occurred
	 */
/*	public static boolean checkAttachedSaltEqualsExistingSalt(String fileName) {
		if (fileName == null) {
			new UnexpectedValueException("String", "fileName", "is null").printDescription();
			return false;
		}
		byte[] existingSalt = KeyDerivation.getSalt();
		if (existingSalt == null) {
			new UnexpectedValueException("byte[]", "existingSalt", "is null").printDescription();
			return false;
		}
		byte[] attachedSalt = null;
		// get and set the attached salt:
		int saltLen = KeyDerivation.getSaltSize();
		byte[] end = null;
		try {
			end = Attachments.getEndBytesOfFile( 
					fileName, 
					Attachments.getFileIdentifierSize() 
					+ saltLen);			
			attachedSalt = new byte[saltLen];
			System.arraycopy(end,  0,  attachedSalt,  0,  saltLen);
		} catch (IOException e) {
			TestLog.e(PeaControl.class, new ExceptionHelper(e).getInfos());
			TestLog.e(PeaControl.class, 
						"###" + // translate here, not in PswDialogView
								PeaProperties.getOsBridge().translate("file_inappropriate")
								+ fileName//chosenFileNames[0] 
								+ PeaProperties.getOsBridge().translate("caused_error")
								+ e.toString() + ", " + e.getMessage() + "\n"
								+ PeaProperties.getOsBridge().translate("attached_salt_not_found"));	
			return false;
		} catch (Exception e) {
			TestLog.e(PeaControl.class, new ExceptionHelper(e).getInfos());
			return false;
		}
		if (Arrays.equals(attachedSalt, existingSalt) == true) {
			return true;
		} else {
			TestLog.e(PeaControl.class, "Salt of file differs from previously set salt: " + fileName);
			return false;
		}
	} */

	
	/**
	 * Derive the key from the password. Performs an initial
	 * hash and derives the key using the selected key derivation function.
	 * The salt must be set before
	 * 
	 * @param pswInputChars		the password
	 * 
	 * @return					the derived key in required size. If this method returns null, 
	 * 							the value derivationError is set
	 */
	public final static byte[] deriveKeyFromPsw(char[] pswInputChars) { 
		
		derivationError = null;

		if (pswInputChars == null) {
			pswInputChars = "no password".toCharArray();
		} else if (pswInputChars.length == 0) {
			pswInputChars = "no password".toCharArray();
		}
		//TestLog.o(getClass(), "PeaControl deriveKeyFromPsw\n"
		//		+ "  pswInputChars: " + new String(pswInputChars));

		byte[] pswInputBytes = Converter.chars2bytes(pswInputChars); 
		//Help.printBytes("PeaControl deriveKeyFromPsw \n  pswInputBytes", pswInputBytes);
		//Help.printBytes("PeaControl deriveKeyFromPsw \n  programRandomBytes", Attachments.getProgramRandomBytes());
		if (pswInputBytes == null) {
			TestLog.ve(PeaControl.class, "Key derivation error");
			TestLog.ve(PeaControl.class, "Error converting char[] to byte[]", 5);
			derivationError = PeaProperties.getVmBridge().translate("unexpected_error") + " (conversion)";
			return null;
		}
		
		if (pswInputBytes.length > 0)  {
			Zeroizer.zero(pswInputChars);
		}
		// prevent using a zero password:
		pswInputChars = null;

		// set algorithms and parameters for hash, KDF and key size
		if (plainTextMode == false) {//getMainWindow() instanceof PswDialogView) {
			
			if ( PswDialogView.isInitializing() == false) {
				if (PeaProperties.isCommandLineMode() == false) {
					// use scheme from FilePanelHolder
					if (FilePanelHolder.getSelectedModel() != null) {
						String schemeToUse = new String(FilePanelHolder.getSelectedModel().getSchemeMarker(), 
								AttachmentHandler.getASCIICharset());
						TestLog.v(PeaControl.class, "scheme to use: " + schemeToUse);
						AlgoParamHandler.readString(schemeToUse, false);
					} else {
						TestLog.e(PeaControl.class, "No files in FilePanelHolder?");
						TestLog.ve(PeaControl.class, "No scheme found in FilePanelHolder", 5);
					}
				} else {
					// scheme was set in CommandLineHandler from first file
			/*		String defaultScheme= PeaProperties.getDefaultAlgoParamString();
					TestLog.v(PeaControl.class, "default scheme: " + defaultScheme);
					AlgoParamHandler.readString(defaultScheme, false); */
				}
			} else {
				// use default algos and parameters
				String defaultScheme= PeaProperties.getDefaultAlgoParamString();
				TestLog.v(PeaControl.class, "default scheme: " + defaultScheme);
				AlgoParamHandler.readString(defaultScheme, false);
			}
		}
		byte[] salt = KeyDerivation.getSalt();
		if (salt == null) {
			if ( PswDialogView.isInitializing() == false) {
				TestLog.ve(PeaControl.class, "No salt found although not initializing", 5);
			}
			TestLog.v(PeaControl.class, "Create new salt...");
			salt =  new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize());
			KeyDerivation.setSalt(salt);
		}
		String saltString =  Converter.bytes2hex(salt);
		TestLog.o(PeaControl.class, "Derive key: "
					+ KeyDerivation.getKdf().getInfoString() + " \n " 		
					+  saltString + " \n "
					+ HashStuff.getHashAlgo().getAlgorithmName() + " - " 
					+ CipherStuff.getCipherAlgo().getAlgorithmName());
		//new UnexpectedValueException("XXXXXXXXXXX","","").printDescription();


		//=============
		//  derive key: 
		//	
		// 1. initial hash, reasons: 
		// - Bcrypt limited password length, 
		// - password remains n RAM in Bcrypt and Scrypt
		// - key file property
		if (PeaProperties.isTestMode()){
			Comparator.checkNullVector(pswInputBytes);
		}
		//TestLog.o(getClass(), Converter.bytes2hex(pswInputBytes) + "   " + Converter.bytes2hex(Attachments.getProgramRandomBytes()));
	//Help.printBytes("PeaControl deriveKeyFromPsw 2  pswInputBytes", pswInputBytes);
		//if (PeaSettings.getKeyFileProperty() == true || version == 1) {
		//TestLog.e(PeaControl.class, "inputChars: " + new String(pswInputBytes));
		//TestLog.e(PeaControl.class, "progRandBytes: " + Converter.bytes2hex(Attachments.getProgramRandomBytes()));
		byte[] pswHash = HashStuff.generateHKDFBytes(pswInputBytes, 
				Attachments.getProgramRandomBytes(), // if no key file property: this is a fixed string
				64); // size of pswHash: 64 bytes
		Zeroizer.zero(pswInputBytes);
		//}
		//Help.printBytes("PeaControl deriveKeyFromPsw \n  pswHash", pswHash);	
		if (PeaProperties.isTestMode()){
			Comparator.checkNullVector(pswHash);
		}

		// 2. derive key from selected KDF: 
		if (KeyDerivation.getKdf() == null) {
			new UnexpectedValueException("KeyDerivation", "getKdf()", "is null").printDescription();
			derivationError = PeaProperties.getVmBridge().translate("unexpected_error") + " (missing KDF)";
			return null;
		}
		if (pswHash == null) {
			new UnexpectedValueException("byte[]", "pswHash", "is null").printDescription();
			derivationError = PeaProperties.getVmBridge().translate("unexpected_error") + " (password hash)";
			return null;
		}
		byte[] keyMaterial = null;
		try {
			//Help.printBytes("pswHash",  pswHash);
			//Help.printBytes("keyMatrerial",  keyMaterial);
//TestLog.e(PeaControl.class, "pswHash: " + Converter.bytes2hex(pswHash));
			keyMaterial = KeyDerivation.getKdf().deriveKey(pswHash);	
			//Help.printBytes("keyMatrerial",  keyMaterial);
//TestLog.e(PeaControl.class, "keyMat: " + Converter.bytes2hex(keyMaterial));
		}  catch (OutOfMemoryError e) {
			if (PswDialogView.getView().isShowing()) {
			TestLog.e(PeaControl.class,   
					"Your system doesn't offer enough memory..." +
					"\n" +	e.getLocalizedMessage() +
					"\nThe key derivation requires more memory than your system provides."
					+ "\n  You can try to run java with more memory by typing: "
					+ "\n    java -Xmx1024M -jar YOUR_PEA.jar");
			derivationError = "Your system doesn't offer enough memory..." +
					"\n" +	e.getLocalizedMessage() +
					"\nThe key derivation requires more memory than your system provides."
					+ "\n  You can try to run java with more memory by typing: "
					+ "\n    java -Xmx1024M -jar YOUR_PEA.jar";
			return null;
			} else {
				TestLog.e(PeaControl.class, 
						"Your system doesn't offer enough memory..." +
						"\n" +	e.getLocalizedMessage() +
						"\nThe key derivation requires more memory than your system provides."
						+ "\n  Set the memory parameter of the key derivation function to a lower value:"
						+ "\n    Menu -> Setting -> crypto -> key derivation -> parameter setting");
				derivationError = "Your system doesn't offer enough memory..." +
						"\n" +	e.getLocalizedMessage() +
						"\nThe key derivation requires more memory than your system provides."
						+ "\n  Set the memory parameter of the key derivation function to a lower value:"
						+ "\n    Menu -> Setting -> crypto -> key derivation -> parameter setting";
				return null;
			}
		}
		
		Zeroizer.zero(pswHash);		
		return keyMaterial;
	}

	/**
	 * Get password, set algorithms and parameters, 
	 * and derive the key. 
	 * The salt must be set before. 
	 * 
	 * @return	the key as an array of bytes
	 */
	protected final byte[] getKeyMaterial() {

		//-------------------------------------------------------------------------------
		// Check if t is alive, if so: wait...
		if (PswDialogView.getView().getThread() != null) {
			if (PswDialogView.getView().getThread().isAlive()) { // this should never happen... 

				TestLog.e(PeaControl.class, "PeaControl: WhileTypingThread still alive!");
				try {
					PswDialogView.getView().getThread().join();
				} catch (InterruptedException e) {
					if (PeaProperties.isTestMode()){
						TestLog.e(PeaControl.class, "PeaControl: Joining thread interrupted");
						e.printStackTrace();
					}
				}
			}		
		}
	
		//---------------------------------------------------------------------
		// get password 
		char[] pswInputChars = null;
		if (PswDialogView.isInitializing() == false){
			if (PeaProperties.isCommandLineMode() == false){
				pswInputChars = PswDialogView.getPassword();	
			} else {
				pswInputChars = CommandLineHandler.getPassword();
			}
		} else {// initializing
			
			if (PeaProperties.getFileType().equals("text file") || PeaProperties.getFileType().equals("image")){
				pswInputChars = Arrays.copyOf(initializedPassword, initializedPassword.length);
			} else {
				pswInputChars = initializedPassword;
			}			
		}

		// set algorithms and parameters:
		if (PswDialogView.isInitializing() == true) {
			// get and set default algorithms and parameters
			AlgoParamHandler.readString(PeaProperties.getDefaultAlgoParamString(), false);
		} else {
			if (PeaProperties.isCommandLineMode() == false) {
				// get and set currently selected algorithms and parameters
				if (FilePanelHolder.getSelectedModel().getSchemeMarker() == null) {
					TestLog.ve(PeaControl.class, "No algorithm-parameter scheme found", 5);
					return null;
				}
				AlgoParamHandler.readString(
						new String(FilePanelHolder.getSelectedModel().getSchemeMarker(), AttachmentHandler.getASCIICharset()), 
						false);
				if (FilePanelHolder.getSelectedModel().getSalt() == null) {
					TestLog.ve(PeaControl.class, "No salt found", 5);
					return null;
				}
				KeyDerivation.setSalt(FilePanelHolder.getSelectedModel().getSalt());
			} else {
				// scheme was set in CommandLineHandler from first file
				//AlgoParamHandler.readString(PeaProperties.getDefaultAlgoParamString(), false);
			}
		}
		
		TestLog.v(PeaControl.class, "Derive key - KDF: " + KeyDerivation.getKdf().getName() + ", Cipher: " + CipherStuff.getCipherAlgo().getAlgorithmName()
				+ ", key size: " + CipherStuff.getKeySize());
		
		//TestLog.e(PeaControl.class, "psw: " + new String(pswInputChars));	
		
		byte[] keyMaterial = deriveKeyFromPsw(pswInputChars);	
		
		if (PeaProperties.getFileType().equals("image") || PeaProperties.getFileType().equals("text file")){
			// do not zeroize
			// the password is needed to decrypt the image or 
			// to decrypt the file (text file)
		} else {
			Zeroizer.zero(initializedPassword);
			initializedPassword = null;
		}
		//TestLog.o(getClass(), "initialized Password: " + initializedPassword);
	
		if (keyMaterial == null) {
			TestLog.e(PeaControl.class, "Key derivation failed" );
			PswDialogView.getView().displayErrorMessages("Key derivation failed.\n"
					+ "Program bug.", true);
			PeaDialog.showMessage(JREProperties.getMainWindow(), 							
					PeaControl.getDerivationError(), 
					PeaProperties.getVmBridge().translate("error"), 
					0);//JOptionPane.ERROR_MESSAGE

//			PswDialogView.clearPassword();
			PswDialogView.enableOkButton();
		}		
		return keyMaterial;
	}

	/**
	 * Initialize required values for execution: hash, cipher, 
	 * KDF, KDF parameters, programRandomBytes, salt, file identifier
	 */
	public final static void initializeVariables() {	
		
		//TestLog.o(getClass(), PeaSettings.getHashAlgo().getAlgorithmName());

		PeaProperties.setVmBridge(new JREBridge());
		HashStuff.setHashAlgo(PeaSettings.getHashAlgo() );
		HashStuff.setHMacAlgo(PeaSettings.getHashAlgo() );
		CipherStuff.setCipherAlgo(PeaSettings.getCipherAlgo() );
		KeyDerivation.setKdf( PeaSettings.getKdfScheme() );
	//	KeyDerivation.setParameters(PeaSettings.getIterations(), PeaSettings.getMemory(), PeaSettings.getParallelization());
	/*	KeyDerivation.getKdf().settCost(PeaSettings.getIterations() );
		KeyDerivation.getKdf().setmCost(PeaSettings.getMemory() );
		KeyDerivation.getKdf().setArg3(PeaSettings.getParallelization());*/
		KeyDerivation.setVersionString(PeaSettings.getVersionString() );
		Attachments.setProgramRandomBytes(PeaSettings.getProgramRandomBytes() );	
		if (PeaSettings.getKeyFileProperty() == false) {
			Attachments.setFileIdentifier(AttachmentHandler.getVersion2ID());
		} else {
			Attachments.setFileIdentifier(PeaSettings.getFileIdentifier() );
		}

		FileTypePanel ftp = FilePanelHolder.getSelectedPanel();
		JREProperties.setTypePanel(ftp);

		// Cloud		
/*		if ( ! PeaProperties.getFileType().equals("file") && ftp != null) {		
			HashMap<String, HashSet<String>> map = CloudControl.createInitialCloudFileMap(
					Converter.bytes2hex(ftp.getSalt()), Converter.bytes2hex(ftp.getAlgoParamString()));
			CloudFilePanel cfp = new CloudFilePanel(JREProperties.getMainWindow(), ftp, map, 
					Converter.bytes2hex(ftp.getSalt()), Converter.bytes2hex(ftp.getAlgoParamString()));
			ftp.setCloudFilePanel(cfp);
			ftp.getCloudFilePanel().updateView();
		}*/
		
//		FilePanelHolder ftpHolder = FilePanelHolder.getInstance();
//		ftpHolder.setVisible(true);		
		// create model, composer and view of files:
	/*	FileModel model = new FileModel();
		FileComposer fc = new FileComposer(model);
		fc.setPlainModus(false);// encrypted files
		FileTypePanel encryptedFileTypePanel =  new FileTypePanel(350, 250, false, false, 
				fc, PswDialogView.getView());
		JREProperties.setTypePanel(encryptedFileTypePanel);	*/


		//AttachmentHandler.initFileNames();
	}
	
	
/*	public static final void initializePassword() {
		NewPasswordDialog.setRandomCollector(true);
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
				PswDialogView.getView(), PeaProperties.getOsBridge().translate("initialization"), null,
				PswDialogView.getView().getLocation());
		dialog.setInitializedPassword( newPswDialog.getDialogInput()
				);
		NewPasswordDialog.setRandomCollector(false);			

		if (newPswDialog.getDialogInput() == null){				
			System.exit(0);
		} else {				
			if (PeaProperties.getFileType().equals("image")) {
				PswDialogView.setMessage("select_image", true);
			}
			dialog.startExecution(false);
		}
	}*/
	
	// If a blank PEA is started first: 
/*	public final byte[] initializeCiphertext(byte[] ciphertext){
		// salt must be set manually and will be attached to the file later
		// in encryption step
		byte[] attachedSalt = new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize());

		if ( ! PeaProperties.getFileType().equals("file") ){//PeaSettings.getBound() == false) {
			KeyDerivation.setSalt(attachedSalt);
		}
		
		// attach salt:
		ciphertext = Attachments.attachBytes(ciphertext, attachedSalt);
		// attach fileIdentifier
		ciphertext = Attachments.attachBytes(ciphertext, Attachments.getFileIdentifier());
		
		return ciphertext;
	} */
	


	//== file functions: ===============================================================	
	// TODO replace with FileModel and remove
	
	
	
	/**
	 * Check a password for a cipher text: 1. compares salt from file with current salt, 
	 * 2. check if the pswIdentifier is the encrypted String "_PEA_FACTORY_ID_". 
	 * The nonce is taken from the cipher text, the cipher text must in the original state,
	 * (not modified: (fileIdentifier, salt and nonce not cut). 
	 * 
	 * @param cipherText	the original cipher text to check
	 * @param key			the key to use for decryption
	 * 
	 * @return				null if the password does not fail, 
	 * 						an error message otherwise
	 */
	public final static String checkPassword(byte[] cipherText, byte[] key) {

		if (cipherText == null) {
			if (PeaProperties.isTestMode()) {
				new UnexpectedValueException("cipherText", "byte[]", "is null").printDescription();
			} 		
			return "Missing ciphertext to check passord...";
		}
		if (key == null) {
			if (PeaProperties.isTestMode()) {
				new UnexpectedValueException("key", "byte[]", "is null").printDescription();
			}
		} else {
			// check for program bug
			if (PeaProperties.isTestMode()){
				Comparator.checkNullVector(key);
			}
		}

		// check the size of the cipher text:
		// TODO algoParam
		if (cipherText == null || cipherText.length < (Attachments.getFileIdentifierSize() 
				+ KeyDerivation.getSaltSize()
					+ Attachments.getNonceSize() 
					+ Attachments.getPswIdentifierSize() ) ){
			return PeaProperties.getVmBridge().translate("unsuitable_ciphertext_len");
		}
		boolean check = AttachmentHandler.validateSaltAlgosFileID(cipherText);
		if (check == false) {
			return PeaProperties.getVmBridge().translate("unexpected_error");
		}
		
		byte[] fileIdentifier = Attachments.getEndOfBytes(cipherText, Attachments.getFileIdentifierSize());
		int version = -1;
		if (Arrays.equals(fileIdentifier, AttachmentHandler.getVersion1ID())) {
			version = 1;
		} else if (Arrays.equals(fileIdentifier, AttachmentHandler.getVersion2ID())) {
			version = 2;
		} else if (PeaSettings.getKeyFileProperty() == true) {
			version = 0;
		} else {
			return PeaProperties.getVmBridge().translate("unexpected_error");
		}
		
		// get pswIdentifier from cipher text:
		byte[] pswIdentifier = new byte[Attachments.getPswIdentifierSize()];
		int pswIdIndex = cipherText.length 
				- Attachments.getFileIdentifierSize() 
				- KeyDerivation.getSaltSize()
				- Attachments.getNonceSize() 
				- Attachments.getPswIdentifierSize();
		if (version == 2) {
			pswIdIndex -= AlgoParamHandler.getAlgoParamStringSize();
		}
		System.arraycopy(cipherText, pswIdIndex, pswIdentifier, 0, Attachments.getPswIdentifierSize());
		byte[] nonce = new byte[ Attachments.getNonceSize()];
		int nonceIndex = cipherText.length 
				- Attachments.getFileIdentifierSize() 					
				- KeyDerivation.getSaltSize()
				- Attachments.getNonceSize();
		if (version == 2) {
			nonceIndex -= AlgoParamHandler.getAlgoParamStringSize();
		}
		System.arraycopy(cipherText, nonceIndex, nonce, 0, Attachments.getNonceSize());
	/*	System.arraycopy(cipherText,  cipherText.length 
					- Attachments.getFileIdentifierSize() 
					- KeyDerivation.getSaltSize()
					- Attachments.getNonceSize() 
					- Attachments.getPswIdentifierSize(),
				pswIdentifier,  0,  pswIdentifier.length);*/
		// get nonce from cipher text:
/*		byte[] nonce = new byte[ Attachments.getNonceSize()];
		System.arraycopy(cipherText,  cipherText.length 
					- Attachments.getFileIdentifierSize() 					
					- KeyDerivation.getSaltSize()
					- Attachments.getNonceSize(),  
				nonce,  0,  nonce.length);*/
		
		boolean result = Attachments.checkPswIdentifier(pswIdentifier, key, nonce);
		
		if (result == true) {
			return null;
		} else {
			return PeaProperties.getVmBridge().translate("password_failed");
		}
	}
	

	
	
	//==========================================
	// Getter & Setter

	public final static void setDialog(PeaControl _dialog) {
		dialog = _dialog;
	}
	public final static PeaControl getDialog() {
		return dialog;
	}

/*	public final static String getErrorMessage() {
		return errorMessage;
	}
	public final static void setErrorMessage(String newErrorMessage) {
		errorMessage = newErrorMessage;
	} */
	
	/**
	 * Set the password from initialization process
	 * to use in key derivation
	 * 
	 * @param psw	the password to set
	 */
	public static void setInitializedPassword(char[] psw){
		initializedPassword = psw;
	}
	/**
	 * @return the encryptedFileTypePanel
	 */
	public FileTypePanel getEncryptedFileTypePanel() {
		return (FileTypePanel)JREProperties.getTypePanel();
	}
	/**
	 * @param encryptedFileTypePanel the encryptedFileTypePanel to set
	 */
/*	public static void setEncryptedFileTypePanel(FileTypePanel encryptedFileTypePanel) {
		PeaControl.encryptedFileTypePanel = encryptedFileTypePanel;
	} */

	/**
	 * Return the file name which can be decrypted with his PEA
	 * 
	 * @return the availableFileNames
	 */
	public static ArrayList<String> getAvailableFileNames() {
		return availableFileNames;
	}
	/**
	 * Set the available file names
	 * 
	 * @param newAvailableFileNames the file names to set
	 */
	public static void setAvailableFileNames(ArrayList<String> newAvailableFileNames) {
		availableFileNames = newAvailableFileNames;
	}
	/**
	 * Remove a file name from the list of available file names
	 * 
	 * @param availableFileNames the availableFileNames to remove
	 * 
	 * @return	true, if the file was present and was removed
	 * 			false, if the file was not present
	 */
	public static boolean removeAvailableFileName(String availableFileName) {
		
		if (PeaProperties.getFileType().equals("file")){
			return false;
		}

		if (PeaControl.availableFileNames.contains(availableFileName)){
			return PeaControl.availableFileNames.remove(availableFileName);
		} else {
			return false;
		}
	}
	/**
	 * @param availableFileNames the availableFileNames to set
	 */
	public static void addAvailableFileName(String availableFileName) {
		
		if (PeaProperties.getFileType().equals("file")){
			return;
		}

		if (PeaControl.availableFileNames.contains(availableFileName)){
//			TestLog.o(getClass(), "file already available: " + availableFileName);
		} else {
			PeaControl.availableFileNames.add(availableFileName);
//			PswDialogView.addAvailableFile(availableFileName, false);// false: not new selected
			//Collections.sort(PeaControl.availableFileNames);
		}
	}
	/**
	 * Get the name of the property file for the specific PEA
	 * without directory (e.g. "file.properties")
	 * 
	 * @return	the name of the property file for the specific PEA
	 * 			without the directory
	 */
	public static String getPeaPropFileName() {
		return peaPropFileName;
	}
	/**
	 * Set the name of the property file for the specific PEA
	 * without the directory (e.g. "file.properties")
	 * 
	 * @param peaPropFileName	the name of the property file for the specific PEA
	 * 							without the directory
	 */
	public static void setPeaPropFileName(String peaPropFileName) {
		PeaControl.peaPropFileName = peaPropFileName;
	}
	public static String getDerivationError() {
		return derivationError;
	}

	/**
	 * Get the current main window: lock frame or 
	 * PswDialogView
	 * 
	 * @return	the current main window
	 */
	/*public static Window getMainWindow() {
		return mainWindow;
	}*/
	/**
	 * Get the current main window: lock frame or 
	 * PswDialogView
	 * 
	 * @param mainWindow	the current main window
	 */
	/*public static void setMainWindow(Window mainWindow) {
		PeaControl.mainWindow = mainWindow;
	}*/
	/**
	 * Check if key was derived and 
	 * decryption was performed
	 * and files are open
	 * 
	 * @return	true if content was decrypted
	 */
	public static boolean isPlainTextMode() {
		return plainTextMode;
	}
	/**
	 * Set if key was derived and 
	 * decryption was performed
	 * and files are open
	 * 
	 * @param filesOpen	true if content was decrypted
	 */
	public static void setPlainTextMode(boolean _contentOpen) {
		PeaControl.plainTextMode = _contentOpen;
	}
}
