package cologne.eck.all_peas.gui;

import java.awt.Window;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.io.File;

import javax.swing.JFrame;

import org.bouncycastle.crypto.BlockCipher;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Converter;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;


@SuppressWarnings("serial")
public abstract class PeaLockFrame extends JFrame {
	
	private boolean changesApplied = true;
	
	/**
	 * Ask for further setting and if not:
	 * Change the password for current used files 
	 * or apply changes in algorithms and parameters:
	 * Ask for a new password, ask for other (remaining) files, 
	 * derive the key, encrypt the files
	 * 
	 * @param owner: PswDialogView or the lock frame
	 * @param algorithmChange	true: ask to make further settings,
	 * 							false for password change only
	 * 
	 * @return	null for success, a localized error message to show otherwise
	 */
	public String changePasswordOrApplyChanges(Window owner, boolean algorithmChange) {
		
		setChangesApplied(false);
		if (algorithmChange == true) {
			String[] options = { PeaProperties.getVmBridge().translate("apply_changes"), //"Apply changes", "make further settings"};
					PeaProperties.getVmBridge().translate("make_further_settings")};

			int answer = PeaDialog.showOptions(owner, 
					PeaProperties.getVmBridge().translate("apply_changes")+"?", PeaProperties.getVmBridge().translate("finalize_setting") + "?", 
					0, 3, options, options[1]);
			if (answer == -1) {// closed
				TestLog.o(getClass(), "Setting cancelled...");
				return null;
			} else if (answer == 1) { // make further settings
				TestLog.o(getClass(), "Make further settings...");
				//SettingMenu.setAlgoParamsBeforeChange(previousAlgoParams);
				return null;			
			} else if (answer == 0) { // apply

				// go on..
			}
		}
		//String oldSaltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
		
		if (owner instanceof PswDialogView) {
			// default algorithms and parameters were changed...
			// create String from default values:
			String newDefaultString = AlgoParamHandler.writeString(true);
			// set this string as default string:
			PeaProperties.setDefaultAlgoParamString(newDefaultString);
		} else if (owner instanceof PeaLockFrame) {
			String currentAlgos = AlgoParamHandler.writeString(false);
			PeaProperties.setAlgoParamString(currentAlgos.getBytes(AttachmentHandler.getASCIICharset()));
			AlgoParamHandler.readString(currentAlgos, false);
		}
		boolean contentWasSaved = false;
		NewPasswordDialog newPswDialog = NewPasswordDialog.getInstance(
				owner, PeaProperties.getVmBridge().translate("change_password"), null,
				owner.getLocation());
		char[] newPsw = newPswDialog.getDialogInput();

		//==============================================
		// new Nonce for every new password
		byte[] nonce = new RandomStuff().createRandomBytes(Attachments.getNonceSize());
		Attachments.setNonce(nonce);
		//==============================================

		try {
			setMessage("... " + PeaProperties.getVmBridge().translate("change_password") + " " +
					PeaProperties.getVmBridge().translate("in_progress")  + "... ", false);
			//messageLabel.setForeground(Color.BLACK);
			//messageLabel.setText("... " + PeaProperties.getVmBridge().translate("change_password") + " " +
			//		PeaProperties.getVmBridge().translate("in_progress")  + "... ");
		} finally {
			// for cloud password
			byte[] previousKey = null;
			BlockCipher previousCipher = null; // cipher might change...
			if (owner instanceof PeaLockFrame && ( ! PeaProperties.getFileType().equals("file"))) {
				previousKey = CipherStuff.getSessionKeyCrypt().getKey();
				if (SettingMenu.getAlgoParamsBeforeChange() != null) {
					previousCipher = AlgoParamHandler.getCipherFromString( new String( SettingMenu.getAlgoParamsBeforeChange(), AttachmentHandler.getASCIICharset()));
				} else {
					previousCipher = CipherStuff.getCipherAlgo();
				}
			}

			byte[] keyMaterial = PeaControl.deriveKeyFromPsw(newPsw);
			if (keyMaterial == null) {
				new UnexpectedValueException("byte[]", "keyMaterial", "is null").printDescription();
				if (previousKey != null) {
					Zeroizer.zero(previousKey);
				}
				return PeaControl.getDerivationError();				
			}
			// key was derived with new algorithms/password:
			setChangesApplied(true);

			if (newPsw != null){
				Zeroizer.zero(newPsw);
			} 
			if (CipherStuff.getSessionKeyCrypt() == null) {
				new UnexpectedValueException("SessionKeyCrypt", "CipherStuff.getSessionKeyCrypt()", "is null").printDescription();
				return PeaProperties.getVmBridge().translate("unexpected_error") + " (SessionKeyCrypt missing)";
			}
			CipherStuff.getSessionKeyCrypt().clearKeys();
			//CipherStuff.resetSessionKeyCrypt();
			CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);
			setMessage("", false);
			//PeaProperties.getOsBridge().translate("done"));
			
			// set properties files for new scheme
			if (owner instanceof PeaLockFrame) {
//				String currentAlgos = AlgoParamHandler.writeString(false);
//				PeaProperties.setAlgoParamString(currentAlgos.getBytes(AttachmentHandler.getASCIICharset()));
				//TestLog.v(PeaLockFrame.class,"Current algo: " + new String(AlgoParamHandler.writeString(false)));

				if ( ! PeaProperties.getFileType().equals("file")) {
					String[] currentFileNames = ((PeaLockFrame) owner).getCurrentFileNames();
					if (currentFileNames != null) {
						String saltAsHex = Converter.bytes2hex(KeyDerivation.getSalt());
						String newAlgoParamAsHex = Converter.bytes2hex(PeaProperties.getAlgoParamString());
						if (saltAsHex != null && newAlgoParamAsHex != null) {
							for (String openedFileName : currentFileNames) {
								if (openedFileName.startsWith(CloudControl.getTmpDir())) {
									String providerName = CloudControl.getProviderNameFromTmpFile(openedFileName);
									String fileNameToRemove = CloudControl.getFileNameFromTmpFile(openedFileName);
									if (providerName != null && fileNameToRemove != null) {
										int removeResult = CloudPropertyHandler.removeFileFromAnyProviderSet(providerName, fileNameToRemove, null);

										if (removeResult >= 0) { // file was removed
											// get password from set, the file was removed from
											String[] subSetRemove = CloudPropertyHandler.getSubSet(providerName, removeResult, null);
											String plainPsw = null;
											if (subSetRemove[2] == null || subSetRemove[2].length() == 0) {
												TestLog.e(PeaLockFrame.class, "Missing password from set, a file was removed from...");
											} else {
												plainPsw = CloudPropertyHandler.decryptPassword(subSetRemove[2], previousKey, previousCipher);
											}
											int extToAdd = CloudPropertyHandler.getSubSetExtension(providerName, saltAsHex, newAlgoParamAsHex, null);
											String[] fileNamesToAdd = {fileNameToRemove};
											if (extToAdd >= 0) { // exists already												
												String addError = CloudPropertyHandler.addFileNamesToProperties(providerName, fileNamesToAdd, 
														saltAsHex, newAlgoParamAsHex);
												if (addError != null) {
													TestLog.e(getClass(), addError);
												}
												// change password:
												String storeError = CloudPropertyHandler.storeEncryptedPasswordFromProvider(
														CloudPropertyHandler.encryptPassword(plainPsw), providerName, 
														saltAsHex, newAlgoParamAsHex, null);
												if (storeError != null) {
													TestLog.e(PeaLockFrame.class, "Could not change cloud password in properties");
												}
											} else {
												// create new set, get password

												int newExt = CloudPropertyHandler.createSubSet(providerName, saltAsHex, newAlgoParamAsHex, 
														plainPsw, null);
												if (newExt == -1) {//failed
													TestLog.e(PeaLockFrame.class, "Could not create new property set for " + providerName);													
												} else {
													// add removed file name to new set
													String addError = CloudPropertyHandler.addFileNamesToProperties(providerName, fileNamesToAdd, saltAsHex, newAlgoParamAsHex);
													if (addError != null) {//failed
														TestLog.e(PeaLockFrame.class, "Could not add file " + fileNameToRemove + " to " + providerName);
													}
												}
											}
										}
									} else {
										TestLog.ve(getClass(), "provider/file name string is null", 5);
										//return PeaProperties.getVmBridge().translate("unexpected_error") + " (missing provider/file name)";
									}
								}
							}
						} else {
							TestLog.ve(getClass(), "Salt or algorithm-parameter string is null", 5);
							return PeaProperties.getVmBridge().translate("unexpected_error") + " (missing salt/algo-string)";
						}
					}
					PeaControl.setAvailableFileNames(Converter.arrayToArrayList(currentFileNames));
					// set new FileTypePanel, CloudFilePanel...
					FileComposer newFc = new FileComposer(new FileModel(false, PeaProperties.getAlgoParamString(), KeyDerivation.getSalt()));
					FileTypePanel newFtp = new FileTypePanel(100,100, true, false, newFc, this);
					// set new panel:
					JREProperties.setTypePanel(newFtp);
					CloudFilePanel newCfp = new CloudFilePanel(owner, newFtp, Converter.bytes2hex(KeyDerivation.getSalt()), 
							Converter.bytes2hex(PeaProperties.getAlgoParamString()));
					// set CloudFilePanel
					newFtp.setCloudFilePanel(newCfp);
					for (String newFileName : currentFileNames) {
						if (newFileName.contains(CloudControl.getTmpDir())) {
							String providerName = CloudControl.getProviderNameFromTmpFile(newFileName);
							String usedFileName = CloudControl.getFileNameFromTmpFile(newFileName);
							// add to used files
							newCfp.getCloudControl().addUsedFileNameToProvider(providerName, usedFileName);
						}
					}
					// Now save the content with the new key
					String saveError = saveCurrentContent(); // TODO implement for all PEAs except Notebook
					if (saveError != null) {
						TestLog.e(PeaLockFrame.class, "Could not save current file - " + saveError);
						TestLog.ve(PeaLockFrame.class, "Could not save current file - " + saveError, 5);
					} else {
						contentWasSaved = true;
						TestLog.o(PeaLockFrame.class, "Content was saved");
					}
				} // not File Lock PEA
			}
			if (previousKey != null) {
				Zeroizer.zero(previousKey);	
			}
		}
		SettingMenu.setAlgoParamsBeforeChange(null);
		if (contentWasSaved == false) {
			// the content is not re-encrypted:
			markUnsavedContentChanges();
		} else {
			markContentIsSaved();
		}		
		return null;
	}
	
	
	/**
	 * Decrypt the content of a file and load the decrypted
	 * content to the current view.
	 * 
	 * @param fileName	the name of the file to decrypt and load
	 * 
	 * @return an error message in the form "file name - error", or null (success)
	 */
	public abstract String decryptAndLoadFileToView(String fileName);
	
	/**
	 * Get the associated file names with path of the
	 * currently opened content (Tab in LockFrameEditor (one file + backup files), 
	 * image in LockFrameImage (one file), 
	 * calendar files in LockFrameCalendar (several files))
	 * 
	 * @return	the file names which is associated with 
	 * 			the currently shown content or null if there is no
	 */
	public abstract String[] getCurrentFileNames();
	
	/**
	 * Set the associated file names with path of the
	 * currently opened content (Tab in LockFrameEditor (one file), 
	 * image in LockFrameImage (one file), 
	 * calendar files in LockFrameCalendar (several files))
	 * 
	 * @param fileNamesToSet	the file names to set
	 */
	public abstract void setCurrentFileNames(String[] fileNamesToSet);

	/**
	 * Set a message in the PeaLockFrame. 
	 * This will be shown on the bottom of the frame. 
	 * Errors are shown in color red. 
	 * 
	 * @param string	the message
	 * @param isError	true, if the message is an error
	 */
	public abstract void setMessage(String string, boolean isError);


	/**
	 * Check if the currently open content 
	 * was modified and not saved manually and ask 
	 * to save the content in this case. 
	 * 
	 * @return	true if content is saved if required,
	 * 			false if unsaved content remains or
	 * 			no unsaved changes exist
	 */
	public abstract boolean checkUnsavedContentChanges();
	
	
	/**
	 * Mark that there are content changes that have not been saved yet.
	 */
	public abstract void markUnsavedContentChanges();
	
	/**
	 * Mark that all current content was saved
	 */
	public abstract void markContentIsSaved();
	
	
	/**
	 * Update the PEA specific model if there is one
	 * with files that are uploaded to the cloud
	 * 
	 * @param files			the file that are uploaded to the cloud
	 * @param providerName	the provider name, the files were uploaded to
	 * 
	 */
	public abstract void updateModelWithCloudFiles(File[] files, String providerName);

	
	/**
	 * Store the current content in a temporary file, that
	 * is used as local backup of a cloud file:
	 *   System.getProperty("java.io.tmpdir") / providerName / fileName
	 *   
	 * @param providerName		the name of the cloud provider
	 * @param fileNameToStore	the file name to store without path
	 * 
	 * @return	an error message, if an error occurred or null
	 */
	public abstract String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore);
	
	/**
	 * Saves the content of all files from getCurrentFileName(). 
	 * This is performed after the password was changed. 
	 * 
	 * @return null for success, an error message otherwise
	 */
	public abstract String saveCurrentContent();


	public boolean isChangesApplied() {
		return changesApplied;
	}


	public void setChangesApplied(boolean changesApplied) {
		this.changesApplied = changesApplied;
	}
	
	/**
	 * Encrypt the list of files with the session key (used in File Lock PEA)
	 * 
	 * @return a list with first element: error message and 
	 *			all following elements: failed files to be deleted
	 */
	//public abstract ArrayList<String>  encryptFiles(ArrayList<File> list);

}
