package cologne.eck.note_pea;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */


/**
 * Dialog to get the password and start the editor pea.
 */


import java.awt.Insets;
import java.awt.Window;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
//import java.util.Set;
import java.util.TimeZone;
import java.util.zip.CRC32;

import java.awt.Color;
import java.awt.Font;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.DefaultStyledDocument;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.control.CommandLineHandler;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.FileRecipient;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.files.FileModel;
//import cologne.eck.all_peas.files.FileModel;
import cologne.eck.all_peas.files.FilePanelHolder;
import cologne.eck.all_peas.files.InvalidPanel;
//import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HtmlFileViewer;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.Toaster;
import cologne.eck.all_peas.vm_specific.JREProperties;

import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;

import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.PropertyHandler;


public final class EditorControl extends PeaControl {

	private static PswDialogView dialogView;
	private static EditorControl pswDialog;
	private static LockFrameEditor lockFrame; 
	private static String peaPropFileName = "editor.properties";
	private static final String[] supportedExtensions = {"rtf"}; 
	
	private static final String PEA_NAME = "Notebook-PEA";
	private static final String VERSION = "1.4 (PEA version 2)";
	private static final String YEAR_OF_PUBLICATION = "2021"; 
	private static final String WEBSITE = "/html/notebook_pea.html";
	private static final String SOURCE_LINK = "http://eck.cologne/peafactory/src/downloads/notebook/notebook-1.4-src.zip"; 
	
	// to show errors and failed files only once: collect errors
	private StringBuilder errorsToShow = new StringBuilder();
	
	private EditorControl() {
		
		pswDialog = this;		
		setDialog(pswDialog);
		dialogView = PswDialogView.getInstance();
		JREProperties.setMainWindow(dialogView);
		PeaControl.setPlainTextMode(false);
	}
	
	public static EditorControl getInstance() {
		EditorControl dialogInstance = null;
		if (pswDialog == null) {			
			dialogInstance = new EditorControl();
		} else {
			// return null
		}
		return dialogInstance;
	}
	
	public static void main(String[] args) {

		PeaProperties.setVersionInfo(PEA_NAME + " " + VERSION);
		PeaProperties.setFileType("text file");		
		PeaControl.setPeaPropFileName(peaPropFileName);
		new PropertyHandler().readAndSetAllPropValues();
		if(args.length > 0) {
			//PeaProperties.setWorkingMode(args[0]);
			boolean runCommands = CommandLineHandler.handleArgs(args);
			TestLog.v(EditorControl.class, "Run in command line mode: " + runCommands);
		}
		if (PeaProperties.isCommandLineMode() == false) {
			PeaControl.initializeVariables();		
			SwingUtilities.invokeLater(new EventDispatchThread());		
		} else {
			TestLog.v(EditorControl.class, "Command line mode...");
		}	
	}

	@Override
	public final LockFrameEditor getLockFrame() {
		return lockFrame;
	}

	@Override
	public void clearSecretValues() {		
		if (CipherStuff.getSessionKeyCrypt() != null) {
			CipherStuff.getSessionKeyCrypt().clearKeys();
		}
	}
	
	private void publish(String message, boolean isError) {

		PswDialogView.setMessage(message, isError, false);
	}
	/**
	 * Reset cursor, enable ok button and optionally
	 * update the view of the password dialog.
	 * This is called in the done method only.
	 * 
	 * @param returnToDialog	true, if the program should return to the 
	 * 							password dialog and check boxes were added or removed,
	 * 							eventually set salt and key are to be cleared
	 * 							(an error occurred)
	 */
	private void cleanUpOnReturn(boolean returnToDialog) {
		// show collected error messages:
		if (errorsToShow != null && errorsToShow.length() > 0) {
			PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getVmBridge().translate("error"), 0);
		}
		// reset errors:
		errorsToShow = new StringBuilder();
		PswDialogView.enableOkButton();
		dialogView.setCursor(CursorManager.getDefaultCursor());
		if (returnToDialog == true) {
//			KeyDerivation.setSalt(null);
			CipherStuff.getSessionKeyCrypt().clearKeys();
			// maybe files were removed...
	//		((FileTypePanel)JREProperties.getTypePanel()).updateFileView();
			FilePanelHolder.updateFileView();
			PswDialogView.clearPassword();
			dialogView.pack();
		}
	}

	@Override
	public void startDecryption() {		
		

		//PeaDialog.toast(JREProperties.getMainWindow(), "Starting...", 3);
		

		String[] fileNames = null;
		
		publish(PeaProperties.getVmBridge().translate("decryption") + "...", false);
		
		dialogView.setCursor(CursorManager.getWaitCursor());
		
		if (PswDialogView.isInitializing() == false) {	

			if ( CipherStuff.getSessionKeyCrypt().isKeySet() == true) {
				TestLog.e(getClass(), "Existing key before KDF...");
				cleanUpOnReturn(true);
			}
			if (KeyDerivation.getSalt() == null) {
				TestLog.e(getClass(), "Missing salt...");
				cleanUpOnReturn(true);
			}
			// derive the key
			// KDF: derive the key from password and salt: 
			publish(PeaProperties.getVmBridge().translate("key_derivation") + "... ", false);
			byte[] keyMaterial = getKeyMaterial();
			if (keyMaterial == null) {
				TestLog.e(EditorControl.class, "key derivation failed");
				if (PeaProperties.isTestMode()) {
					new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
				}
				errorsToShow.append("\n" + PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)");					
				//PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getOsBridge().translate("error"), 0);
				publish(PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)", true);
				cleanUpOnReturn(true);
				return;
			} else {
				CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);
			}

			FileRecipient frw = new FileRecipient();
			fileNames = frw.getFileNames();
			//TestLog.v(getClass(), "fileNames: ");for(String s : fileNames)System.out.println(s);

			// From FileRecetionWorker: collect to show later with other errors
			if (frw.getCollectedErrorsToShow() != null && frw.getCollectedErrorsToShow().length() > 0) {
				errorsToShow.append( frw.getCollectedErrorsToShow());
			}
			if (fileNames == null) {
				publish(PeaProperties.getVmBridge().translate("no_valid_file_found"), true);
				cleanUpOnReturn(true);
				return;
			}
			
			HashMap<String, HashSet<String>> removeMap = frw.getProviderAndFilesToRemove();
			if (removeMap != null && removeMap.size() > 0) {
				Iterator<HashMap.Entry<String, HashSet<String>>> it = removeMap.entrySet().iterator();
				while (it.hasNext()) {
					HashMap.Entry<String, HashSet<String>> pair = it.next();
					TestLog.ve(this.getClass(), "Remove provider: " + pair.getKey());
					HashSet<String> set = pair.getValue();
					for (String s : set) {
						TestLog.ve(this.getClass(), "Remove file: " + s);
						this.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl()
						.removeInitialFileOfProvider(pair.getKey(), s);
						// show as invalid file
						InvalidPanel.addInvalidFileCheckBox( pair.getKey() + "-" + s, PeaProperties.getVmBridge().translate("error") 
								+ " - " + (String) pair.getKey() + FileModel.getInvalidMarker());
						//((FileTypePanel)JREProperties.getTypePanel()).addInvalidFileCheckBox( pair.getKey() + "-" + s, PeaProperties.getVmBridge().translate("error") 
						//		+ " - " + (String) pair.getKey() + FileModel.getInvalidMarker());
					}
					//TestLog.v(this.getClass(), pair.getKey() + " = " + pair.getValue());
					it.remove(); // avoids a ConcurrentModificationException
				}
			}

			//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
			//>>>>>>>>> Checks and Preparations: 
			//>>>>>>>>>		return if no valid file found
			//>>>>>>>>> 	derive the key if not already derived
			//>>>>>>>>>		check the key against the first valid file
			//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
/*
			byte[] keyMaterial;
			// KDF: derive the key from password and salt: 
			if ( CipherStuff.getSessionKeyCrypt().getKey() == null) {
				publish(PeaProperties.getVmBridge().translate("key_derivation") + "... ", false);
				keyMaterial = getKeyMaterial();
				if (keyMaterial == null) {
					TestLog.e(EditorControl.class, "key derivation failed");
					if (PeaProperties.isTestMode()) {
						new UnexpectedValueException("keyMaterial", "byte[]", "is null").printDescription();
					}
					errorsToShow.append("\n" + PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)");					
					//PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getOsBridge().translate("error"), 0);
					publish(PeaProperties.getVmBridge().translate("unexpected_error") + " (kdf)", true);
					cleanUpOnReturn(true);
					return;
				}
			} else { 
				keyMaterial = CipherStuff.getSessionKeyCrypt().getKey();
			}
			*/

			// check the key against all files:
			// read cipher text from file and check pswIdentifier
			//int firstFileIndex = 0;
/*			boolean passowrdFittingFileFound = false;
			int len = fileNames.length;
			for (int i = 0; i < len; i++) {
				if(new File(fileNames[i]).isFile()) {
					//firstFileIndex = i;
					byte[] cipherText = ReadResources.readExternFile(fileNames[i]);
					if (cipherText == null) {
						if (PeaProperties.isTestMode()) {
							new UnexpectedValueException("cipherText", "byte[]", "is null for " + fileNames[i]).printDescription();
						} 		
						errorsToShow.append("\n" + fileNames[i] + PeaProperties.getVmBridge().translate("unexpected_error") + " (read ct)");					
						publish(PeaProperties.getVmBridge().translate("unexpected_error") + " (read ct)", true);
						continue;
					}
					// check password identifier
					String error = PeaControl.checkPassword(cipherText, keyMaterial);
					if (error != null) {
						// password failed: display error
			//			PswDialogView.clearPassword();
						TestLog.e(getClass(), error + " (password check)");x
						publish(error, true);
						errorsToShow.append("\n" + fileNames[i] + "\n" + error + "\n(password check)");					
						//PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getOsBridge().translate("error"), 0);
						continue;
					} else { // suitable file found
						passowrdFittingFileFound = true;
						break;
					}
				}
			}
			if (passowrdFittingFileFound == false) {
				cleanUpOnReturn(true);
				return;
			}

			// do not decrypt, just store the key
			CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);				*/		
			PeaControl.setAvailableFileNames(new ArrayList<String>(Arrays.asList(fileNames)));
			
		} else { // initializing...
			
			publish(PeaProperties.getVmBridge().translate("init_new_file") + "... ", false);
			// set new salt to attach to file 
			byte[] lastSalt = null;//PeaProperties.getLastSalt();

			// take salt from last selected FileTypePanel if there is one
			if (KeyDerivation.getSalt() != null && FilePanelHolder.getFileNumber() > 0) { 
				lastSalt = KeyDerivation.getSalt();
			} else {
				// get last salt from properties:
				byte[] propSalt = PeaProperties.getLastSalt();
				if (propSalt != null) {
					lastSalt = propSalt;
				}
			}			
			if (KeyDerivation.getSalt() == null) { // should never happen: salt was set in initializeNewFiles
				if (lastSalt != null) {
					// ask
					int result = PeaDialog.showQuestion(
							//FileControl.dialogView, 
							PswDialogView.getView(),
							//		PeaProperties.getOsBridge().translate("add_new_files_when_decrypted") + "\n" +
							PeaProperties.getVmBridge().translate("open_files_together"), 
							PeaProperties.getVmBridge().translate("init_new_file"),
							0, 1);
					if (result == 0){ // yes  ok
						KeyDerivation.setSalt(lastSalt);
					} else {
						KeyDerivation.setSalt(
								new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
					}
				} else {
					// FilePanelHolder should created a salt... so this should not happen
					if (KeyDerivation.getSalt() == null) {
						KeyDerivation.setSalt(
								new RandomStuff().createRandomBytes(KeyDerivation.getSaltSize()));
					}
				}
			}
			// derive key from password dialog:
			byte[] keyMaterial = getKeyMaterial();
			// store the key to use later:
			CipherStuff.getSessionKeyCrypt().storeKey(keyMaterial);			
			
			// salt and nonce are set, key derived, file was prepared, 
			// model was set in initializeNewFile
			fileNames = new String[PeaControl.getAvailableFileNames().size()];
//TestLog.v(getClass(), "fileNames: ");for(String s : fileNames)System.out.println(s);
			TestLog.v(this.getClass(), "Initialization...");					
		}		
		

		// change background color
		PswDialogView.setUI(false);

		// lockFrame: 
		if (lockFrame == null) {
			new PropertyHandler().setSpecificPropValue();
			// TODO ?? dimension??
//			Dimension dim = PeaProperties.getFrameDim();//CalendarSetting.getViewDimension();
			//			WeekPanel weekPanel = new WeekPanel(LocalDate.now(),dim);//new Dimension( dim.width - 90, dim.height - 150));
			//			LockFrameCalendar.setView(weekPanel);
			//lockFrame = LockFrameEditor.getInstance(pswDialog, dialogView.getLocation());
			setLockFrame(LockFrameEditor.getInstance(pswDialog, dialogView.getLocation()));
			
		}
		
		//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
		//>>>>>>>>> Iterate through file names: 
		//>>>>>>>>>		decrpyt content
		//>>>>>>>>> 	check for duplicates and sort them out
		//>>>>>>>>>		add calendars to lockFrame

		// this list contains all successfully decrypted file names whether or not they are double
		// and wether or not they contain valid calendars
		//HashMap<String, HashSet<PCalendar>> validFileWithNotesMap = new HashMap<String, HashSet<PCalendar>>();

		
		// set the CloudFileModel with encrypted files
		// add cloud files to model
		for (String validFileName : fileNames) {
			if (validFileName.contains(CloudControl.getTmpDir())) {
				//TestLog.v(EditorControl.class, "Add to CloudFileModel: " + validFileName, 5);
				this.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().addUsedFileNameToProvider(
						CloudControl.getProviderNameFromTmpFile(validFileName), 
						CloudControl.getFileNameFromTmpFile(validFileName));
			}
		}

		
		// collect the duplicates to set associated file names of duplicates later
		ArrayList<HashSet<String>> listOfDuplicateSets = new ArrayList<HashSet<String>>();
		
		checkDuplicateContent(fileNames, //validFileWithNotesMap, 
				listOfDuplicateSets);
		

		//PswDialogView.getView().setVisible(false);		

		lockFrame.setVisible(true);// ANZEIGE-ERROR
		JREProperties.setMainWindow(lockFrame);
		PeaControl.setPlainTextMode(true);
		
//		Set<String> tabs = NotebookModel.getTabNames();
		ArrayList<HashSet<String>> setList = NotebookModel.getSetList();
		for (HashSet<String> set : setList) {
			for (String s : set) {
				if (lockFrame.decryptAndLoadFileToView(s) == null) {// no error
					break;
				} else {
					// try next file name
				}
			}
		}
	/*	for (String tabName : tabs) {
			lockFrame.decryptAndLoadFileToView(tabName);
		}*/
		
		// set the ChangeListener when all tab were added:
		lockFrame.setChangeListener();
		
		EntropyPool.getInstance().stopCollection();
		
		if (errorsToShow != null && errorsToShow.length() > 0) {
			// show all collected errors
			PeaDialog.showMessage(dialogView, errorsToShow, PeaProperties.getVmBridge().translate("error"), 0);
			// reset errors
			errorsToShow = new StringBuilder();
		}
		// Add successfully added files to cloud.properties
		if (this.getEncryptedFileTypePanel() != null) { //  null in initialization
			CloudFilePanel cfp = this.getEncryptedFileTypePanel().getCloudFilePanel();
			if (cfp != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();
				if (providers != null && providers.length > 0) {
					int providersLen = providers.length;
					for (int i = 0; i < providersLen; i++) {

						String[] usedFiles = cfp.getCloudControl().getUsedFileNamesOfProvider(providers[i]);
						String error = CloudPropertyHandler.addFileNamesToProperties(providers[i], usedFiles,
								Converter.bytes2hex(KeyDerivation.getSalt()), Converter.bytes2hex(PeaProperties.getAlgoParamString()));
						if (error != null) {
							TestLog.ve(this.getClass(), error);
						}

					}
				}
			}
		}
		PswDialogView.getView().setVisible(false);		
		cleanUpOnReturn(false);
		if (lockFrame != null) {
			lockFrame.setDocChangeUnsaved(false);
			setLockFrame(lockFrame);
		} else {
			TestLog.e(getClass(), "lockFrame is null");
			TestLog.ve(getClass(), "lockFrame is null",  5);
		}
		StringBuilder toastBuilder = new StringBuilder();
		for (HashSet<String> set : listOfDuplicateSets) {
			if (set.size() > 1) {
				toastBuilder.append(PeaProperties.getVmBridge().translate("synchronized" ) + ": \n");
				for (String s : set) {
					toastBuilder.append("   " + s + "\n");
				}
				toastBuilder.append("\n");
			}
		}
		if (toastBuilder.length() > 1) { // show synchronized files
			Toaster.toast(JREProperties.getMainWindow(), new String(toastBuilder), 3);
		}
	}
	
	private static synchronized void setLockFrame(LockFrameEditor lfe) {

		LockFrameEditorMenu.setLockFrame(lfe);
		EditorControl.lockFrame = lfe;
	}
	/**
	 * Ask the user which file to open if file names are
	 * equal, but file size or checksum are different
	 * 
	 * @param file		the file with different size or checksum
	 * @param list		the list of files, stored before with same file name
	 * @param fileNamesToCheck	the list of file names, containing the index
	 * 
	 * @return	true: replace existing files with file and open this file, 
	 * 			false: ignore the file
	 */
	private boolean replaceListedFiles(File file, 
			ArrayList<long[]> list, String[] fileNamesToCheck) {
		
		// TODO cloud: from cloud instead of absolute path
		// TODO create panel with button group file names
		// and label: size,lastMod
		String message = PeaProperties.getVmBridge().translate("files_same_name_different_content") + ".\n"
				+ PeaProperties.getVmBridge().translate("which_one_open") + "\n";
				//+ "Existing:\t" + existingList + "\n"
				//+ "File:\t" + file.lastModified() + " " + file.length();
		String newFileName = null;
		if (file.getAbsolutePath().contains(CloudControl.getTmpDir())) {
			newFileName = PeaProperties.getVmBridge().translate("cloud") + " " 
					+ CloudControl.getProviderNameFromTmpFile(file.getAbsolutePath()) + ": "
							+ file.getName();
		} else {
			newFileName = file.getAbsolutePath();
		}
		
		String[] options = {PeaProperties.getVmBridge().translate("previous_files") + ": ", newFileName};

		long lastMod = file.lastModified();
		String ldt = "";
		if (lastMod > 0L) {
			ldt = PeaProperties.getVmBridge().translate("last_mod") + ": "
					+ LocalDateTime.ofInstant(Instant.ofEpochMilli(file.lastModified()), 
                TimeZone.getDefault().toZoneId()).toString() + ", ";  
		}
		String infoNewFile = ldt + (file.length() / 1024) + " KB";
		
		String infoPreviousFiles = "";
		for (long[] valuesInList : list) {
			String fileNameInList = fileNamesToCheck[(int)valuesInList[0]];
			if (fileNameInList.contains(CloudControl.getTmpDir())) {
				fileNameInList = PeaProperties.getVmBridge().translate("cloud") + " " 
						+ CloudControl.getProviderNameFromTmpFile(fileNameInList) + ": "
								+ file.getName();
			}
			File previousFile = new File(fileNameInList);
			long lastMod2 = previousFile.lastModified();
			String ldt2 = "";
			if (lastMod2 > 0L) {
				ldt2 = PeaProperties.getVmBridge().translate("last_mod") + ": "
						+ LocalDateTime.ofInstant(Instant.ofEpochMilli(file.lastModified()), 
	                TimeZone.getDefault().toZoneId()).toString() + ", ";  
			}
			infoPreviousFiles += fileNameInList + "\n"
					+ ldt2 + (previousFile.length() / 1024) + " KB" +"\n";
		}
		
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
		
		
		JPanel textFieldPanel = new JPanel();
		textFieldPanel.setLayout(new BoxLayout(textFieldPanel, BoxLayout.LINE_AXIS));
		JTextArea textField = new JTextArea();		
		textField.setBorder(null);
		int[] plainVals = PeaProperties.getPlainPeaColor();
		textField.setBackground(new Color(plainVals[0], plainVals[1], plainVals[2]));
		//textField.setLineWrap(true);
		message = message.replaceAll("\\\\n", System.getProperty("line.separator"));
		textField.setText(message);
		textField.setEditable(false);
		textFieldPanel.add(textField);
		textFieldPanel.add(Box.createHorizontalGlue());
		p.add(textFieldPanel);
		p.add(Box.createVerticalStrut(10));
		
	    ButtonGroup group = new ButtonGroup();
	    
		JPanel rb1Panel = new JPanel();
		rb1Panel.setLayout(new BoxLayout(rb1Panel, BoxLayout.LINE_AXIS));
	    JRadioButton rb1 = new JRadioButton(options[0]);
	    rb1.setSelected(true); // select initial value
	    rb1.setFont(new Font(Font.SANS_SERIF, Font.BOLD, PeaProperties.getFontSize() ));
	    rb1Panel.add(rb1);
	    rb1Panel.add(Box.createHorizontalGlue());
	    p.add(rb1Panel);
		group.add(rb1);
		
		JTextArea previousInfos = new JTextArea(infoPreviousFiles);
		previousInfos.setBackground(new Color(plainVals[0], plainVals[1], plainVals[2]));
		previousInfos.setBorder(null);
		p.add(previousInfos);
	    
		JPanel rb2Panel = new JPanel();
		rb2Panel.setLayout(new BoxLayout(rb2Panel, BoxLayout.LINE_AXIS));
	    JRadioButton rb2 = new JRadioButton(options[1]);
	    rb2.setSelected(true); // select initial value
	    rb2.setFont(new Font(Font.SANS_SERIF, Font.BOLD, PeaProperties.getFontSize() ));
	    rb2Panel.add(rb2);
	    rb2Panel.add(Box.createHorizontalGlue());
	    p.add(rb2Panel);
	    
		group.add(rb2);
		
		JPanel labelPanel = new JPanel();
		labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.LINE_AXIS));
		JLabel newFileLabel = new JLabel(infoNewFile);
		//newFileLabel.setBackground(Color.WHITE);
		labelPanel.add(newFileLabel);
		labelPanel.add(Box.createHorizontalGlue());
		p.add(labelPanel);
		
		JOptionPane.showMessageDialog(JREProperties.getMainWindow(), p, null, 3,
				null);//PswDialogView.getImageIcon());
		if (rb1.isSelected()) {
			return false;
		} else if (rb2.isSelected()) {
			return true;
		} else { // close
			return false;
		}	
	}
	
	
	/**
	 * Check the list of file names for duplicates:
	 * check 1. file size, 2. file name and  3. checksum of encrypted content.
	 * Others than Calendar Lock PEA, there is no decryption here. 
	 * If file names are equal, but not size/checksum: Ask which one to open
	 * 
	 * @param fileNamesToCheck				the file names to check
	 * @param decryptedFileWithCalendarsMap		map of successfully decrypted files (whether or not duplicate and contain calendars)  
	 * 								with RTF texts in that file
	 * @param listOfDuplicateSets	a list of sets of duplicate file names (each set contains file names that are duplicate)
	 */
	private void checkDuplicateContent(	String[] fileNamesToCheck, 
			//HashMap<String, HashSet<PCalendar>> decryptedFileWithCalendarsMap, 
			ArrayList<HashSet<String>> listOfDuplicateSets) {
		
		// iterate over file names: 
		int len = fileNamesToCheck.length;
		
		// Generate a map of files to be opened
		// key: file name without path
		// value: 	0: index of file in fileNamesToCheck
		//			1: file size of files
		//			2: checksum of encrypted files
		HashMap<String, ArrayList<long[]>> fileNameIndexMap = new HashMap<String, ArrayList<long[]>>();

		CRC32 crc = new CRC32();
		//long newestLastMod = 0;
		
		for (int i = 0; i < len; i++) {

			File file = new File( fileNamesToCheck[i] );
//	TestLog.v(this.getClass(), "*** check: "+fileNamesToCheck[i] + " size: " + file.length());
			if ( file.isFile()) { // checks also existence

				long fileSize = file.length(); // get file length
				// access, pswIdentifier... already checked in FileRecipient
				crc.update(ReadResources.readExternFile(fileNamesToCheck[i]));
				long  checksum1 = crc.getValue(); // get crc checksum
				crc.reset();
				// 1. check file name
				String nameOfFile = file.getName();
				if (fileNameIndexMap.containsKey(nameOfFile)) {
					// values of first file in list (values  of size and checksum are equal)
					long[] values = fileNameIndexMap.get(nameOfFile).get(0);
					if (fileSize == values[1]) {
						
						if (checksum1 == values[2]) {
							// add new file to key:
							long[] newValues = {i, fileSize, checksum1};
							fileNameIndexMap.get(nameOfFile).add(newValues);
						} else {
							// ask user, which file should be opened
							boolean replace = replaceListedFiles(file, 
									fileNameIndexMap.get(nameOfFile), fileNamesToCheck);
							if (replace == true) {
								// remove existing key
								fileNameIndexMap.remove(nameOfFile);
								// create new key:
								long[] newValues = {i, fileSize, checksum1};
								ArrayList<long[]> list = new ArrayList<long[]>();
								list.add(newValues);
								// put a new entry into the map
								fileNameIndexMap.put(nameOfFile, list);
							} else {
								TestLog.v(this.getClass(), "Rejected: " + fileNamesToCheck[i]);
							}
						}
						
					} else {
						// ask user, which file should be opened
						boolean replace = replaceListedFiles(file, 
								fileNameIndexMap.get(nameOfFile), fileNamesToCheck);
						if (replace == true) {
							// remove existing key
							fileNameIndexMap.remove(nameOfFile);
							// create new key:
							long[] newValues = {i, fileSize, checksum1};
							ArrayList<long[]> list = new ArrayList<long[]>();
							list.add(newValues);
							// put a new entry into the map
							fileNameIndexMap.put(nameOfFile, list);
						} else {
							TestLog.v(this.getClass(), "Rejected: " + fileNamesToCheck[i]);
						}
					}
				} else {
					// create new key:
					long[] newValues = {i, fileSize, checksum1};
					ArrayList<long[]> list = new ArrayList<long[]>();
					list.add(newValues);
					// put a new entry into the map
					fileNameIndexMap.put(nameOfFile, list);
				}
			} else { //  file exists
				new UnexpectedValueException("File", "file", "does not exist: " + fileNamesToCheck[i]).printDescription();
			}
		} // for loop fileNamesToCheck
		
		// add files to model and handle cloud files
		for (HashMap.Entry<String, ArrayList<long[]>> entry : fileNameIndexMap.entrySet()) {
			
//			  String fileNameInMap = entry.getKey();
			  ArrayList<long[]> valueList = entry.getValue();

			  // collect all equal file names with path
			  HashSet<String> set = new HashSet<String>();

			  for (long[] val : valueList) {
				  String fileNameWithPath = fileNamesToCheck[ (int)val[0] ];
				  set.add(fileNameWithPath);
					// add provider to CloudControl
					if (fileNameWithPath.contains(CloudControl.getTmpDir())) {
						String providerName = CloudControl.getProviderNameFromTmpFile(fileNameWithPath);
						String propFileName = CloudControl.getFileNameFromTmpFile(fileNameWithPath);
			//			CloudPropertyHandler.setSingleProperty(providerName + "_files", propFileName);
						// TODO test
						String[] filesToAdd = {propFileName};
						String error = CloudPropertyHandler.addFileNamesToProperties(providerName, filesToAdd,
								Converter.bytes2hex(KeyDerivation.getSalt()), Converter.bytes2hex(PeaProperties.getAlgoParamString()));
						if (error != null) {
							TestLog.e(EditorControl.class, error);
							TestLog.ve(EditorControl.class, "Adding file names to properties failed: " + error,5);
						}
					}
			  }
			  listOfDuplicateSets.add(set);
			  NotebookModel.addFileNameSet(set);
		}
	}
	/**
	 * Quick check if a byte array starts with "{\\rtf1\\ansi".
	 * This should only be used for decrypted files from
	 * Notebook PEA, otherwise the file might start with comments
	 * 
	 * @param bytes	the byte array to check
	 * 
	 * @return	true if byte array starts with "{\\rtf1\\ansi"
	 */
/*	protected static boolean quickCheckRTF(byte[] bytes) {
		String rtfString = "{\\rtf1\\ansi";
		byte[] rtfBytes = rtfString.getBytes(PeaProperties.getCharset());
		byte[] firstPart = new byte[rtfBytes.length];
		System.arraycopy(bytes, 0, firstPart, 0, rtfBytes.length);
		if (Arrays.equals(rtfBytes,firstPart)) {
			return true;
		} else {
			return false;
		}
	}*/
	
	// not used here:
	@Override
	public void preComputeInThread() {}
	@Override
	public String[] getSelectedFileNames() {
		return FilePanelHolder.getSelectedPanel().getValidLocalFileNames(true, false);
		//return ( (FileTypePanel) JREProperties.getTypePanel()).getValidFileNames(true, false);
	}

	/**
	 * Other than File Lock PEA and Image Lock PEA, only
	 * one single file with extension .rtf can be initialized. 
	 * If a file with the same name exists, a warning and an 
	 * option to break is shown. If possible, the user is asked 
	 * to make the new file compatible to previous files 
	 * (using the same salt). 
	 */
	@Override
	public void initializeNewFiles() {		
		// get default values from properties
		String algoParamString = PeaProperties.getDefaultAlgoParamString();//new String(PeaProperties.getAlgoParamString(), AttachmentHandler.getASCIICharset());
		TestLog.v(EditorControl.class, "Default algorithm-parameter string: " + algoParamString);
		// set to use these settings
		AlgoParamHandler.readString(algoParamString, false);
		// set algo parameter string to use
		PeaProperties.setAlgoParamString(algoParamString.getBytes(AttachmentHandler.getASCIICharset()));//AlgoParamHandler.writeString().getBytes(AttachmentHandler.getASCIICharset()));		

		
		Window owner = JREProperties.getMainWindow();

	    PeaFileChooser chooser = new PeaFileChooser(owner);

	    chooser.customizeFileChooser(
	    		PeaProperties.getVmBridge().translate("init_new_file"),
	    		PeaProperties.getVmBridge().translate("select_location_and_file_name"), 
	    		false);// no hidden files option
	    
		FileNameExtensionFilter txtfilter = new FileNameExtensionFilter(
                    "rich text format", "rtf");	
		
        chooser.setFileFilter(txtfilter);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
	   // chooser.setCurrentDirectory(new File("/home/Documents"));

	    int retrival = chooser.showOpenDialog(owner);
	    if (retrival == JFileChooser.APPROVE_OPTION) {		    	
	    	
	    	File selectedFile = chooser.getSelectedFile();
	    	
	    	// append file extension:
	    	if ( ! selectedFile.getPath().endsWith(".rtf")){
	    		selectedFile = new File(selectedFile.getPath() + ".rtf");	
	    	}
	    	
	    	///// for later use: when initializing new file in lockFrame:
			if (NotebookModel.fileNameExists(selectedFile.getAbsolutePath()) == true) {
				TestLog.e(EditorControl.class, "File to initialize is already listed: " + selectedFile.getAbsolutePath());
				PeaDialog.showMessage(owner, PeaProperties.getVmBridge().translate("file_already_open"), 
						PeaProperties.getVmBridge().translate("error"), 0);
				return;
			} 
	    	
	    	// Warning if file exists: 			    	
	    	if ( selectedFile.exists()){
	    		int n = JOptionPane.showConfirmDialog(owner,
	    				PeaProperties.getVmBridge().translate("overwrite_existing_file"), 
	    				PeaProperties.getVmBridge().translate("warning"), 
	    			JOptionPane.OK_CANCEL_OPTION,
	    			JOptionPane.WARNING_MESSAGE);
	    		if (n != JFileChooser.APPROVE_OPTION){ // cancel, close
	    			//return;
	    			System.exit(0);
	    		}
	    	}		    		    	
	    	try {
	    		owner.setCursor(CursorManager.getWaitCursor());

//	    		setSaltForNewFiles();
	    		// create an empty file:
	    		if (selectedFile.exists() == false) {
	    			boolean success = true;
	    			try {
	    				success = selectedFile.createNewFile();
	    			} catch (IOException e) {
	    				TestLog.e(EditorControl.class, "Creation of new empty file failed: " + selectedFile.getAbsolutePath());
	    				//PswDialogView.setMessage( PeaProperties.getOsBridge().translate("read_write_failed") , true, false);
	    				if (owner instanceof PswDialogView) {
	    					PswDialogView.setMessage( PeaProperties.getVmBridge().translate("read_write_failed") , true, false);
	    				} else {
	    					((PeaLockFrame) lockFrame).setMessage(PeaProperties.getVmBridge().translate("read_write_failed"), true);
	    				}
	    				JOptionPane.showMessageDialog(owner,
	    						PeaProperties.getVmBridge().translate("read_write_failed")
	    						+ "\n" + selectedFile.getAbsolutePath(),
	    						PeaProperties.getVmBridge().translate("error") + ": Creating new file",
	    						JOptionPane.ERROR_MESSAGE);			
	    				return;
	    			}
	    			if (success == false) {
	    				TestLog.e(EditorControl.class, "Creation of new empty file failed: " + selectedFile.getAbsolutePath());
	    				if (owner instanceof PswDialogView) {
	    					PswDialogView.setMessage( PeaProperties.getVmBridge().translate("read_write_failed") , true, false);
	    				} else {
	    					((PeaLockFrame) lockFrame).setMessage(PeaProperties.getVmBridge().translate("read_write_failed"), true);
	    				}
	    				JOptionPane.showMessageDialog(owner,
	    						PeaProperties.getVmBridge().translate("read_write_failed")
	    						+ "\n" + selectedFile.getAbsolutePath(),
	    						PeaProperties.getVmBridge().translate("error") + ": Creating new file",
	    						JOptionPane.ERROR_MESSAGE);			
	    				return;	    			
	    			}
	    		}

	    		boolean saveSuccess = ContentUtilities.saveContent(getKeyMaterial(), 
	    				"\n".getBytes(PeaProperties.getCharset()), null, selectedFile.getPath(), true);
	    		if (saveSuccess == true) {

	    			HashSet<String> set = new HashSet<String>();
	    			set.add(selectedFile.getAbsolutePath());
	    			NotebookModel.addFileNameSet( set);
	    			
	    			// add to path file:
	    			PathFileManager.addFileNameToPathFile(selectedFile.getAbsolutePath());

	    			// store session key:				
	    			CipherStuff.getSessionKeyCrypt().storeKey(getKeyMaterial());
	    		} else {
    				JOptionPane.showMessageDialog(owner,
    						PeaProperties.getVmBridge().translate("unexpected_error")
    						+ "\n" + selectedFile.getAbsolutePath() + " (save)",
    						PeaProperties.getVmBridge().translate("error"),
    						JOptionPane.ERROR_MESSAGE);			
    				return;	 
	    		}

	    	} finally {
	    		owner.setCursor(CursorManager.getDefaultCursor());
	    	}

	    } else {
	    	TestLog.e(EditorControl.class, "file chooser canceled or closed");
	    	System.exit(0);
	    }  
	}

	@Override
	public void openEncryptedFiles() {
		
		boolean fileAdded = AttachmentHandler.addEncryptedFiles(JREProperties.getMainWindow(), 
				PeaProperties.getVmBridge().translate("open_encrypted_file"), 
				true, false);// hidden files, do not check listed files
		
		if (fileAdded == false) { // no file was added
			TestLog.e(EditorControl.class, "no file was added");
			return;
		}

		
		boolean validFileFound = FilePanelHolder.getSelectedModel().checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getOsBridge().translate("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true, false);
		}		
/*

		FileComposer encryptedComposer = ((FileTypePanel)JREProperties.getTypePanel()).getFileComposer();
		FileModel encryptedModel = encryptedComposer.getFileModel();
		boolean added = encryptedComposer.addAction(((FileTypePanel)JREProperties.getTypePanel()),
				PeaProperties.getVmBridge().translate("select_files_to_encrypt"), 
				true, false);// hidden files, do not check listed files
		if (added == false) { // no file was added
			TestLog.e(EditorControl.class, "no file was added");
			return;
		}

		boolean validFileFound = encryptedModel.checkForValidFile();
		if (validFileFound == false) {
			//PswDialogView.setMessage(PeaProperties.getOsBridge().translate("no_valid_file_found"), true);
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
		} else {
			// clear errorMessageLabel
			PswDialogView.setMessage("   ", true, false);
		}		*/
	}
	@Override
	public String getVersion() {
		return VERSION;
	}
	@Override
	public String getYearOfPublication() {
		return YEAR_OF_PUBLICATION;
	}
	@Override
	public String getWebsite() {
		return WEBSITE;
	}
	@Override
	public String getSourceLink() {
		return SOURCE_LINK;
	}

	@Override
	public boolean checkFormat(File file, boolean easyCheck, boolean middleCheck, boolean expensiveCheck) {

		String fileName = file.getAbsolutePath();
		if (easyCheck == true) {
			String extension = "";
			int i = fileName.lastIndexOf('.');
			if (i > 0) {
			    extension = fileName.substring(i+1);
			}
			if (! extension.toUpperCase().equals("RTF")) {
		    	if (PeaProperties.isTestMode()) {
					TestLog.e(getClass(), PeaProperties.getVmBridge().translate("unsuitable_rtf_file") 
							+ ": "  + fileName); 
					TestLog.e(EditorControl.class, "File extension is not .rtf");
		    	}
				return false;
			} 
		}
		if (middleCheck == true) { // TODO test
			// check if file content starts with {\rtf1 in the first line
			BufferedReader brTest;
			String firstLine = "";
			try {
				brTest = new BufferedReader(new FileReader(file));
				firstLine = brTest .readLine();
			} catch (FileNotFoundException e) {
				TestLog.e(EditorControl.class, e.getLocalizedMessage());
				TestLog.ve(EditorControl.class, e.toString(), 5);
			} catch (IOException e) {
				TestLog.e(EditorControl.class, e.getLocalizedMessage());
				TestLog.ve(EditorControl.class, e.toString(), 5);
			}
		    if (firstLine == null || ! firstLine.contains("rtf1")) {
		    	if (PeaProperties.isTestMode()) {
				TestLog.e(getClass(), 
						PeaProperties.getVmBridge().translate("unsuitable_rtf_file") 
						+ ": "  + fileName); 
				TestLog.e(EditorControl.class, "Unsuitable control sequenz: Could not find sequenz rtf1 in first line");
		    	}
		    	return false;
		    }
		}
		if (expensiveCheck == true) {
			Object obj = LockFrameEditor.deserialize(ReadResources.readExternFile(fileName));		

			if (! (obj instanceof DefaultStyledDocument) ) { // wrong psw (malformed file) not all wrong files
				if (PeaProperties.isTestMode()) {
					TestLog.e(getClass(), 
							PeaProperties.getVmBridge().translate("unsuitable_rtf_file") 
							+ ": "  + fileName);
					TestLog.e(EditorControl.class, "Unsuitable style of document");
				}
				//return false;
			}
		}
		return true;
	}

	@Override
	protected byte[] getPlainBytes() {
		return lockFrame.getPlainBytes();
		//return null;
	}

	@Override
	public String setPeaPropValues(Properties prop, InputStream inputStream) {
		String errorMessage = null;
		try {
			// Default font: ======================================
			String fontStyleString = prop.getProperty("font_style");
			if (fontStyleString != null) {
				try {
					NotebookSetting.setFontStyle(Integer.parseInt(fontStyleString));
				} catch (Exception e) {
					new UnexpectedValueException("prop", "Properties", "is not digit: " + prop.getProperty("font_style")).printDescription();
				}
			}			
			String fontFamily = prop.getProperty("font_family");
			if (fontFamily != null) {
				NotebookSetting.setFontFamily(fontFamily);
			}
			String fontSizeString = prop.getProperty("text_font_size");
			if (fontSizeString != null) {
				try {
					NotebookSetting.setFontSize(Integer.parseInt(fontSizeString));
				} catch (Exception e) {
					new UnexpectedValueException("prop", "Properties", "is not digit: " + prop.getProperty("text_font_size")).printDescription();
				}
			}
			//
			int r = 0, g = 0, b = 0;
			String rString = prop.getProperty("text_background_color_r");
			if (rString != null) {
				try {
					r = Integer.parseInt(rString);
				} catch (Exception e) {
					r = NotebookSetting.getTextBackground().getRed();
					new UnexpectedValueException("rString", "String", "is not digit: " + rString).printDescription();
				}
			} else {
				r = NotebookSetting.getTextBackground().getRed();
			}
			String gString = prop.getProperty("text_background_color_g");
			if (gString != null) {
				try {
					g = Integer.parseInt(gString);
				} catch (Exception e) {
					g = NotebookSetting.getTextBackground().getGreen();
					new UnexpectedValueException("gString", "String", "is not digit: " + gString).printDescription();
				}
			} else {
				g = NotebookSetting.getTextBackground().getGreen();
			}
			String bString = prop.getProperty("text_background_color_b");
			if (bString != null) {
				try {
					b = Integer.parseInt(bString);
				} catch (Exception e) {
					b = NotebookSetting.getTextBackground().getBlue();
					new UnexpectedValueException("bString", "String", "is not digit: " + bString).printDescription();
				}
			} else {
				b = NotebookSetting.getTextBackground().getBlue();
			}
			NotebookSetting.setTextBackground(new Color(r, g, b));
			
			
			String rFontString = prop.getProperty("font_color_r");
			if (rFontString != null) {
				try {
					r = Integer.parseInt(rFontString);
				} catch (Exception e) {
					r = NotebookSetting.getFontColor().getRed();
					new UnexpectedValueException("rFontString", "String", "is not digit: " + rFontString).printDescription();
				}
			} else {
				r = NotebookSetting.getFontColor().getRed();
			}
			String gFontString = prop.getProperty("font_color_g");
			if (gFontString != null) {
				try {
					g = Integer.parseInt(gFontString);
				} catch (Exception e) {
					g = NotebookSetting.getFontColor().getGreen();
					new UnexpectedValueException("gFontString", "String", "is not digit: " + gFontString).printDescription();
				}
			} else {
				g = NotebookSetting.getFontColor().getGreen();
			}
			String bFontString = prop.getProperty("font_color_b");
			if (bFontString != null) {
				try {
					b = Integer.parseInt(bFontString);
				} catch (Exception e) {
					b = NotebookSetting.getFontColor().getBlue();
					new UnexpectedValueException("bFontString", "String", "is not digit: " + bFontString).printDescription();
				}
			} else {
				b = NotebookSetting.getFontColor().getBlue();
			}
			NotebookSetting.setFontColor(new Color(r, g, b));

					
		} catch ( Exception e){
			TestLog.e(getClass(), e + ", " + e.getMessage());
			new UnexpectedValueException("","","set pea properties").printDescription();
			errorMessage = e.toString() + ", " + e.getLocalizedMessage();
		}
		return errorMessage;

	}

	@Override
	public String updatePeaPropValues(Properties prop) {//, InputStream inputStream) {
		String errorMessage = null;
		try {
			prop.setProperty("font_style", "" + NotebookSetting.getFontStyle());
			prop.setProperty("font_family", NotebookSetting.getFontFamily());
			prop.setProperty("text_font_size", "" + NotebookSetting.getFontSize());

			prop.setProperty("text_background_color_r","" + NotebookSetting.getTextBackground().getRed());
			prop.setProperty("text_background_color_g","" + NotebookSetting.getTextBackground().getGreen());
			prop.setProperty("text_background_color_b","" + NotebookSetting.getTextBackground().getBlue());

			prop.setProperty("font_color_r", "" + NotebookSetting.getFontColor().getRed());
			prop.setProperty("font_color_g", "" + NotebookSetting.getFontColor().getGreen());
			prop.setProperty("font_color_b", "" + NotebookSetting.getFontColor().getBlue());
			
					
		} catch ( Exception e){
			TestLog.e(getClass(), e + ", " + e.getMessage());
			errorMessage = e.toString() + ", " + e.getLocalizedMessage();
		}
		return errorMessage;	
	}
	
	private static class EventDispatchThread implements Runnable{

		@Override
		public void run() {
			PswDialogView.setUI(true);
			// customize button margin
			UIManager.put("Button.contentMargins", new Insets( 5,9,5,9) );		
			pswDialog = EditorControl.getInstance();		
			EditorControl.dialogView.setVisible(true);
			setDialog(pswDialog);			
			if (FilePanelHolder.getFileNumber() == 0) {
				PswDialogView.setMessage(PeaProperties.getVmBridge().translate("open_or_initialize"), true, false);
			}
			if (new File("resources" + File.separator + "pea.properties").exists() == false) { // first time
				HtmlFileViewer hfv = new HtmlFileViewer("first_time_guide", dialogView);
				hfv.requestFocus();
			}

			//lockFrame.setDocChangeUnsaved(false);			
		}		
	}

	@Override
	public String getPeaName() {
		return PEA_NAME;
	}

	@Override
	public String[] getSupportedExtensions() {
		return supportedExtensions;
	}
}	