package cologne.eck.note_pea;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.awt.Window;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import javax.accessibility.AccessibleContext;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.Painter;
import javax.swing.SwingUtilities;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;




@SuppressWarnings("serial")
class TabComponent extends JScrollPane 
		implements ActionListener, // for close button of tab
		MouseListener { // for CopyCutPastePopoup
	
	/**
	 * The JTabbedPane where this instance is added to
	 */
	private final JTabbedPane tabPane;
	
	/**
	 * The text pane associated with this instance 
	 */
	private JTextPane textPane;
	
	/**
	 * The names of the files containing the encrypted content
	 * and associated with this instance
	 */
	private HashSet <String> associatedFileNamesWithPath;
	
	/**
	 * The maximal length of the title of the tab. 
	 * The title is a substring of the file name
	 */
	private static int maxTitleLen = 25;
	
	/**
	 * The file name without path displayed on top of the tab as title
	 */
	private String tabTitle;

	/**
	 * The file name with path, that is shown as title and
	 * where the content is loaded from
	 */
	private String shownFileNameWithPath;
	
	/**
	 * The tool tip text of the tab: All file names with path
	 * from the set if necessary as HTML-string
	 */
	private String toolTipText = null;

	/**
	 * TabComponent is a JScrollPane which is used for the JTabbedPane
	 * of the Notebook PEA. Each TabComponent is associated with a 
	 * file name of an encrypted text file and a JTextPane to display 
	 * the file content. 
	 * If the tab is selected, the title of the tab contains a pea icon 
	 * and the background is green, otherwise the background is gray. 
	 * 
	 * @param tabPane	the JTabbedPane that contains this TabComponent
	 * @param fileNameWithoutPath	the name of the associated file 
	 */
	protected TabComponent(final JTabbedPane _tabPane, HashSet<String> fileNames) {
		
        if (_tabPane == null) {
            throw new NullPointerException("TabComponent: JTabbedPane is null");
        }
        this.tabPane = _tabPane;

        if (fileNames == null || fileNames.size() == 0) {
        	// do not allow null file name in constructor
        	//throw new NullPointerException("TabComponent: file name is null");
        	new UnexpectedValueException("HashSet<String>", "fileNames", "is null or empty").printDescription();
        	return;
        }
        
        this.associatedFileNamesWithPath = fileNames;
        shownFileNameWithPath = preferLocalFileName(fileNames);
        tabTitle = getTitleOfTab( shownFileNameWithPath );
        this.setBorder(BorderFactory.createEmptyBorder());
        if (associatedFileNamesWithPath.size() == 1) {
        	this.toolTipText = shownFileNameWithPath;
        } else if (associatedFileNamesWithPath.size() > 1) {
        	StringBuilder toolTipTextBuilder = new StringBuilder();
        	toolTipTextBuilder.append("<html>");
        	for (String s : associatedFileNamesWithPath) {
        		if (s.contains(CloudControl.getTmpDir())) {
        			String cloudProvider = CloudControl.getProviderNameFromTmpFile(s);
        			String fileNameWithoutPath = CloudControl.getFileNameFromTmpFile(s);
        			toolTipTextBuilder.append("<p><b>" + cloudProvider + "</b>: " + fileNameWithoutPath + "</p>");
        		} else {
        			toolTipTextBuilder.append("<p>" + s + "</p>");
        		}
        	}
        	toolTipTextBuilder.append("</html>");
        	this.toolTipText = new String(toolTipTextBuilder);
        	this.setToolTipText(new String(this.toolTipText));
        }
        this.setName(tabTitle);
        int fontSize = PeaProperties.getFontSize();//PswDialogView.getFontsize();
        
        // create the JTextPane:
        textPane = new JTextPane();	
        textPane.setName(this.tabTitle);
        textPane.setOpaque(true);
		textPane.setEditable(true);
		UIManager.put("TextPane[Enabled].backgroundPainter", new BackgroundPainter());//new Painter<JComponent>() {
/*		    @Override
		    public void paint(Graphics2D g, JComponent comp, int width, int height) {
		        g.setColor(NotebookSetting.getTextBackground());
		        g.fillRect(0, 0, width, height);
		    }
		}); */
		UIDefaults defaults = new UIDefaults();
		  textPane.putClientProperty("Nimbus.Overrides", defaults);
		  textPane.putClientProperty("Nimbus.Overrides.InheritDefaults", true);
		textPane.setBackground(NotebookSetting.getTextBackground());//new Color(231, 231, 231) );
		textPane.setFont(new Font(
				NotebookSetting.getFontFamily(), 
				NotebookSetting.getFontStyle(), 
				NotebookSetting.getFontSize()));
		
		textPane.setForeground(NotebookSetting.getFontColor());
		
		textPane.setDragEnabled(true);
		textPane.setToolTipText("CTRL + ...  R (undo), W (redo), X (cut), C (copy), V (paste)");
		// set the JTextPane as component for this instance: 
		this.setViewportView(textPane);
		
		// add this instance of TabComponent to the tabPane:
		tabPane.addTab(
				this.tabTitle,	
				null,
				this, 
				this.toolTipText);
		
		// set the close button for the tab and LIGHT_GRAY as background color
		// to indicate that the file is untouched
		JPanel tabTitleDisplay = new JPanel();
		tabTitleDisplay.setName(tabTitle);
		tabTitleDisplay.setToolTipText(toolTipText);
		JLabel titleLabel = new JLabel();
		titleLabel.setToolTipText(toolTipText);
		titleLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
		// cut the file name and display the last part with leading ellipsis (…)
		String labelText = null;
		labelText = this.tabTitle;//"..." + this.fileNameWithoutPath;
		titleLabel.setText(labelText);
		titleLabel.addMouseListener(this);
		JLabel iconLabel = new JLabel();
		//iconLabel.setIcon(IconManager.loadScaledIcon("locked.png", "closed lock", 20, 20));
		iconLabel.setIcon(IconManager.loadIcon("locked.png", PeaProperties.getVmBridge().translate("closed_padlock"), 
				null, PeaProperties.getVmBridge().translate("encrypted_indicator") ) );
		tabTitleDisplay.add( iconLabel);
		tabTitleDisplay.setBackground(Color.LIGHT_GRAY.brighter());
		titleLabel.setBackground(Color.LIGHT_GRAY.brighter());
		tabTitleDisplay.add( titleLabel);
		JButton closeButton = new JButton("x");
		closeButton.addActionListener(this);
		tabTitleDisplay.add(closeButton);
		tabPane.setTabComponentAt(tabPane.indexOfComponent(this), tabTitleDisplay);
	}

	
	/**
	 * Display text as DefaultStyledDocument in the TextPane
	 * of this instance. 
	 * 
	 * @param plainBytes	the text to display as an array of bytes
	 * @param textPane		the TextPane to display
	 */
	private final void displayText(byte[] plainBytes, JTextPane textPane) {
		
		Object obj = LockFrameEditor.deserialize( plainBytes);		
		
		if (! (obj instanceof DefaultStyledDocument) ) { // wrong psw (malformed file) not all wrong files
			TestLog.e(TabComponent.class, "unsuitable document, not a styled document");
			TestLog.e(TabComponent.class, "This file is not a valid rft file. This program can not open this file.");
			
			PswDialogView.setMessage(PeaProperties.getVmBridge().translate("unsuitable_rtf_file"), true, false);
			
			JOptionPane.showMessageDialog(PswDialogView.getView(),
					PeaProperties.getVmBridge().translate("unsuitable_rtf_file"),
					PeaProperties.getVmBridge().translate("error"),
				    JOptionPane.ERROR_MESSAGE);

			textPane.setText(new String(plainBytes,PeaProperties.getCharset()));

		} else {
			DefaultStyledDocument doc = (DefaultStyledDocument) obj;
			textPane.setStyledDocument(doc);	
		}						
		Zeroizer.zero(plainBytes);
		SimpleAttributeSet attr = new SimpleAttributeSet();
		StyleConstants.setForeground(attr, NotebookSetting.getFontColor());
		//JTextPane textPane = LockFrameEditor.getFocusedTextPane();
		textPane.setCharacterAttributes(attr, false);
		((LockFrameEditor)PeaControl.getDialog().getLockFrame()).setDocChangeUnsaved(false);
	}
	
	
	/**
	 * Read the associated file, decrypt the content 
	 * and set the decrypted content as text in the text pane. 
	 * Possible errors are file == null (no dialog displayed), 
	 * file access (dialog), different salt (dialog), and 
	 * decryption/password failure (dialog). 
	 *  
	 * @return	a localized error message to show if the decryption failed
	 * 			or null if the decryption was successful
	 */
	protected String setText() {
		
		// do not allow to type text until method was successful
		textPane.setEditable(false);
		
		String errorMessage = null;		
		if (this.shownFileNameWithPath == null) {
			// file name was set null before because of decryption error
			TestLog.e(TabComponent.class, "TabComponent: no file name " );
			errorMessage = PeaProperties.getVmBridge().translate("no_valid_file_found");
			new UnexpectedValueException("String", "fileName", "is null").printDescription();
			return errorMessage;
		}		
		
		try {
			// set the wait cursor: 
			((Window) PeaControl.getDialog().getLockFrame()).setCursor(CursorManager.getWaitCursor());

			String fileName = shownFileNameWithPath;
			if (fileName == null) {
				fileName = preferLocalFileName(associatedFileNamesWithPath);
			}
			byte[] cipherBytes = ReadResources.readExternFile(fileName);
			if (cipherBytes == null) {			
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				TestLog.e(TabComponent.class, "TabComponent: Access failed: " + fileName);
				errorMessage = PeaProperties.getVmBridge().translate("no_access");
		/*		JOptionPane.showMessageDialog(this,
						PeaProperties.getVmBridge().translate("no_access")
					    + "\n" + fileName,
					    PeaProperties.getVmBridge().translate("error") + ": " + PeaProperties.getVmBridge().translate("decryption"),
					    JOptionPane.ERROR_MESSAGE);			*/
				return errorMessage;
			}
			
			// check identifier, algos and salt:
			boolean valid = AttachmentHandler.validateSaltAlgosFileID(cipherBytes);
			if (valid == false) {
				return PeaProperties.getVmBridge().translate("unexpected_error");
			}
//			int idSize = Attachments.getFileIdentifierSize();
/*			byte[] checkSalt = new byte[KeyDerivation.getSaltSize()];
			System.arraycopy(cipherBytes, cipherBytes.length - Attachments.getFileIdentifierSize() 
					- KeyDerivation.getSaltSize(), 
					checkSalt, 0, KeyDerivation.getSaltSize());
			if (Comparator.compare(checkSalt, KeyDerivation.getSalt()) == false){
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				TestLog.e(TabComponent.class, "TabComponent: Salt failed: " + fileName);
				errorMessage = PeaProperties.getVmBridge().translate("salt_differs_from_first_selected_file");
				JOptionPane.showMessageDialog(this,
						PeaProperties.getVmBridge().translate("salt_differs_from_first_selected_file")
					    + "\n" + fileName,
					    PeaProperties.getVmBridge().translate("error") + ": " + PeaProperties.getVmBridge().translate("decryption"),
					    JOptionPane.ERROR_MESSAGE);			
				return errorMessage;
			}*/
			
			// check password and mac
			byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
			if (plainBytes == null){
				// decryption failed
	//			LockFrameEditor lfe = ((LockFrameEditor) PeaControl.getDialog().getLockFrame());
				// reset the cursor:
				((Window) PeaControl.getDialog().getLockFrame()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

				errorMessage = CipherStuff.getErrorMessage();
				if (errorMessage == null) {
					errorMessage = PeaProperties.getVmBridge().translate("error") + ": " + PeaProperties.getVmBridge().translate("decryption");
				}
				
				TestLog.e(TabComponent.class, "TabComponent: Decryption failed: " + errorMessage + ", " + fileName);
		/*	JOptionPane.showMessageDialog(SwingUtilities.getWindowAncestor(this),
						PeaProperties.getVmBridge().translate(errorMessage)				   
					    + "\n  " + fileName,
					    PeaProperties.getVmBridge().translate("error") + ": " + PeaProperties.getVmBridge().translate("decryption"),
					    JOptionPane.ERROR_MESSAGE);
*/
				return errorMessage;
			}
			// allow to set text
			textPane.setEditable(true);
			displayText(plainBytes, textPane);
			
			// Set the default settings: 
			// get existing attribute set
			MutableAttributeSet attributes = textPane.getInputAttributes();
			// font family, size and style
			StyleConstants.setFontFamily(attributes, NotebookSetting.getFontFamily());
	        StyleConstants.setFontSize(attributes, NotebookSetting.getFontSize());
	        if (NotebookSetting.getFontStyle() == Font.BOLD) {
	        	StyleConstants.setBold(attributes, true);
	        } else if (NotebookSetting.getFontStyle() == Font.ITALIC) {
	        	StyleConstants.setItalic(attributes, true);
	        } 
	        // font color
	        StyleConstants.setForeground(attributes, NotebookSetting.getFontColor());
	        // get the document
	        StyledDocument document = textPane.getStyledDocument();
	        // Replace the style at end of document
	        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
	        
	        ((LockFrameEditor)PeaControl.getDialog().getLockFrame()).setDocChangeUnsaved(false);
			
		} finally {
			// reset the cursor:
			((Window) PeaControl.getDialog().getLockFrame()).setCursor(CursorManager.getDefaultCursor());
		}
		return errorMessage;
	}
	/**
	 * Set the title of a TabComponent and a close button. 
	 * If the tab is focused, the title contains a pea icon 
	 * and the background is green, otherwise the background
	 * is gray. 
	 * 
	 * @param iconified		true if the tab should display an icon
	 * 						(should be marked as selected/focused)
	 */
	protected void setTabTitle(boolean iconified) {
		
		//TestLog.v(this.getClass(), "TabComp setTabTitle: " + this.fileName + ", focus: " + iconified);
		
		if (tabPane.indexOfComponent(this) == -1){
			TestLog.e(TabComponent.class, "set tab title failed: Tab does not exist");
			return;
		}
		
		if (this.associatedFileNamesWithPath == null || this.associatedFileNamesWithPath.isEmpty()) {
			new UnexpectedValueException("HashSet<String>", "associatedFileNamesWithPath", "is null").printDescription();
		}

		if (this.toolTipText == null) {
	        if (associatedFileNamesWithPath.size() == 1) {
	        	this.setToolTipText(shownFileNameWithPath);
	        } else if (associatedFileNamesWithPath.size() > 1) {
	        	StringBuilder toolTipTextBuilder = new StringBuilder();
	        	toolTipTextBuilder.append("<html>");
	        	for (String s : associatedFileNamesWithPath) {
	        		toolTipTextBuilder.append("<p>" + s + "</p>");
	        	}
	        	toolTipTextBuilder.append("</html>");
	        	this.toolTipText = new String(toolTipTextBuilder);
	        	this.setToolTipText(new String(this.toolTipText));
	        }
		}
		JPanel tabTitleDisplay = new JPanel();
		tabTitleDisplay.setToolTipText(toolTipText);
		JLabel titleLabel = new JLabel();
		titleLabel.addMouseListener(this);
		titleLabel.setToolTipText(toolTipText);
		// file name with leading ellipsis (…)
		String labelText = tabTitle;

		titleLabel.setText(labelText);
		titleLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 
				PeaProperties.getFontSize()));//PswDialogView.getFontsize()));
		
		JLabel iconLabel = new JLabel();
		if (iconified == true) {

			Image img = PswDialogView.getImage().getScaledInstance(20,20,Image.SCALE_DEFAULT);
			iconLabel.setIcon(new ImageIcon(img));
			tabTitleDisplay.add( iconLabel);
			int[] plainVals = PeaProperties.getPlainPeaColor();
			tabTitleDisplay.setBackground(new Color(plainVals[0], plainVals[1], plainVals[2]));//new Color(229, 255, 231));
			// maybe not necessary: 
			tabPane.setBackgroundAt(tabPane.indexOfComponent(this), new Color(plainVals[0], plainVals[1], plainVals[2]));//new Color(229, 255, 231));
			titleLabel.setBackground(new Color(plainVals[0], plainVals[1], plainVals[2]));//new Color(229, 255, 231));
			
		} else {

			//iconLabel.setIcon(IconManager.loadScaledIcon("locked.png", "closed lock", 20, 20));
			iconLabel.setIcon(IconManager.loadIcon("locked.png", PeaProperties.getVmBridge().translate("closed_padlock"), 
					null, PeaProperties.getVmBridge().translate("encrypted_indicator") ) );
			tabTitleDisplay.add( iconLabel);
			
			//titleLabel.setPreferredSize(new Dimension(200, 30) );
			tabTitleDisplay.setBackground(new Color(216, 216, 216));//Color.LIGHT_GRAY);
			// maybe not necessary: 
			tabPane.setBackgroundAt(tabPane.indexOfComponent(this), new Color(216, 216, 216));
			titleLabel.setBackground(new Color(216, 216, 216));//Color.LIGHT_GRAY);
			titleLabel.addMouseListener(this);
		}
		tabTitleDisplay.add( titleLabel);//new JLabel(this.fileName ));
		JButton closeButton = new JButton("x");
		closeButton.addActionListener(this);
		tabTitleDisplay.add(closeButton);
		tabPane.setTabComponentAt(tabPane.indexOfComponent(this), tabTitleDisplay);;
	}

	/**
	 * Get one file name from the set of associated
	 * file names, prefer a local file to load the content
	 * 
	 * @param set	the set of associated file names
	 * 
	 * @return	one file of the set (local file preferred)
	 */
	private String preferLocalFileName(HashSet<String> set) {
		String chosenFileName = null;
		Iterator<String> iter = set.iterator();
		while (iter.hasNext()) {
			String nextFileName = iter.next();
			//if (chosenFileName == null) {
			chosenFileName = nextFileName;
			if ( ! chosenFileName.contains(CloudControl.getTmpDir())) {
				// local file
				break;
			} else {
				// continue;
			}
		}
		return chosenFileName;
	}
	/**
	 * Get the file name to display as title of the tab component
	 * 
	 * @param fileN	the file name with path to create the title from
	 * 
	 * @return	the title of the tab to show
	 */
	private String getTitleOfTab(String fileN) {
		int len = fileN.length();
		if (len <= maxTitleLen) {
			return fileN;
		}
		String lastPart = fileN.substring(len - maxTitleLen, len);
		int firstSepIndex = lastPart.indexOf(File.separator);
		if (firstSepIndex == -1) { // extreme long file name, no separator
			firstSepIndex = 0; // last part of file name
		}
		return "..." + lastPart.substring(firstSepIndex);			
	}
	/**
	 * Close the current tab, remove from model 
	 * this does not move the focus to the next tab
	 */
	private void closeTab() {
		// the tab to close = this instance 
		int closingTabIndex = tabPane.indexOfComponent(this);
		// the actually selected tab in the tabPane
		int selectedTabIndex = tabPane.indexOfComponent(tabPane.getSelectedComponent());
		if (closingTabIndex == selectedTabIndex) {
			// handle unsaved changes:
			( (LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedContentChanges();
		}
		if (associatedFileNamesWithPath == null) {

		} else {
			Iterator<String> iter = associatedFileNamesWithPath.iterator();
			ArrayList<String> list = PeaControl.getAvailableFileNames();
			while(iter.hasNext()) {
				String nextFileName = iter.next();
				PathFileManager.addFileNameToPathFile(nextFileName); // checks encryptedFileName
				list.remove(nextFileName);
			}
			PeaControl.setAvailableFileNames(list);
		}

		// remove from model:
		NotebookModel.removeFromMap(shownFileNameWithPath);
		
		// remove the this tab from view:
		if (tabPane.indexOfComponent(this) != -1) {
			// first set previously selected tab selected again:
			TabComponent lastFocusTab = LockFrameEditor.getCurrentTab();
			if (lastFocusTab != null && (tabPane.indexOfComponent(lastFocusTab) != -1)) {
				tabPane.setSelectedComponent(LockFrameEditor.getCurrentTab());
			}
			if (tabPane.indexOfComponent(this) != -1
					&& tabPane.getTabCount() > 0) {
				try { // if no valid tab is left, this will throw an IndexOutOfBoundExcetion
					tabPane.remove(tabPane.getComponentAt(closingTabIndex));
				} catch (IndexOutOfBoundsException e) {
					TestLog.e(getClass(), e.toString() + ", closing tab failed: " + e.getMessage());
				}
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		
		closeTab();

/*		// the tab to close = this instance 
		int closingTabIndex = tabPane.indexOfComponent(this);
		// the actually selected tab in the tabPane
		int selectedTabIndex = tabPane.indexOfComponent(tabPane.getSelectedComponent());
		if (closingTabIndex == selectedTabIndex) {
			// handle unsaved changes:
			( (LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedTextChanges();
		}
		if (associatedFileNamesWithPath == null) {
			//new UnexpectedValueException("HashSet<String>", "associatedFileNamesWithPath", 
			//		"is null: " + tabTitle).printDescription();			
			//		return;
			//	}
		} else {
			Iterator<String> iter = associatedFileNamesWithPath.iterator();
			ArrayList<String> list = PeaControl.getAvailableFileNames();
			while(iter.hasNext()) {
				String nextFileName = iter.next();
				PathFileManager.addFileNameToPathFile(nextFileName); // checks encryptedFileName
				list.remove(nextFileName);
			}
			PeaControl.setAvailableFileNames(list);
		}

		// remove from model:
		NotebookModel.removeFromMap(shownFileNameWithPath);
		
		// remove the this tab from view:
		if (tabPane.indexOfComponent(this) != -1) {
			// first set previously selected tab selected again:
			TabComponent lastFocusTab = LockFrameEditor.getCurrentTab();
			if (lastFocusTab != null && (tabPane.indexOfComponent(lastFocusTab) != -1)) {
				tabPane.setSelectedComponent(LockFrameEditor.getCurrentTab());
			}
			if (tabPane.indexOfComponent(this) != -1
					&& tabPane.getTabCount() > 0) {
				try { // if no valid tab is left, this will throw an IndexOutOfBoundExcetion
					tabPane.remove(tabPane.getComponentAt(closingTabIndex));
				} catch (IndexOutOfBoundsException e) {
					TestLog.e(getClass(), e.toString() + ", closing tab failed: " + e.getMessage());
				}
			}
		} */
	}
	
	@Override
	public void mouseClicked(MouseEvent mce) { // cut-copy-paste

		//if(mce.getModifiers() == InputEvent.BUTTON3_MASK){ //rechte Maustaste
		if (SwingUtilities.isRightMouseButton(mce)) {
			Component invoker = mce.getComponent();
			CopyCutPastePopup popup = CopyCutPastePopup.getInstance(invoker);
			
			if (popup.isVisible()){

				popup.setVisible(false);
			} else {

				try {
					popup.setLocation(mce.getLocationOnScreen());				
					popup.setVisible(true);				
				} catch (NullPointerException npe) {
					TestLog.e(TabComponent.class, "Popup in EditorLockFrame: NullPointer");
				}	
			}
		} else { // left mouse button: change tab focus
			
			// remove highlighter from search
			textPane.getHighlighter().removeAllHighlights();

			if (mce.getSource() instanceof JLabel) {
				if ( LockFrameEditor.getCurrentTab() != null
						&& ! LockFrameEditor.getCurrentTab().equals(this)) {

					try { 
						// set the wait cursor: 
						this.setCursor(CursorManager.getWaitCursor());
						tabPane.setSelectedComponent(this);
					} catch (IllegalArgumentException e) {
						TestLog.ve(getClass(), e.getLocalizedMessage(), 5);
					} finally {
						// reset the cursor:
						this.setCursor(CursorManager.getDefaultCursor());
					} 
				}
			}
		}
	}
	@Override
	public void mouseEntered(MouseEvent arg0) {}
	@Override
	public void mouseExited(MouseEvent arg0) {}
	@Override
	public void mousePressed(MouseEvent arg0) {}
	@Override
	public void mouseReleased(MouseEvent arg0) {}

	/**
	 * Get the file names associated with this 
	 * TabComponent as set
	 * 
	 * @return  the associated file names as set
	 */
	public HashSet<String> getFileNames() {
		return this.associatedFileNamesWithPath;
	}
	/**
	 * Set/replace the current set of associated 
	 * file names with this TabComponent
	 * 
	 * @param  the new file names as set
	 */
	public void setFileName(HashSet<String> newFileNames) {
		this.associatedFileNamesWithPath = newFileNames;
	}
	/**
	 * 
	 * @param newFileName
	 * @return
	 */
	/*public boolean addFileName(String newFileName) {
		return this.associatedFileNamesWithPath.add(newFileName);
	}*/
	
	public String getFileNameWithoutPath() {
		return this.shownFileNameWithPath;
	} 

	
	/**
	 * Get the JTextPane, displaying the decrypted file content, 
	 * associated with this instance of TabCompoent
	 * 
	 * @return	the JTextPane associated with this TabComponent
	 */
	protected JTextPane getTextPane(){
		return textPane;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleTabComponent();
        }
        return accessibleContext;
    }
    
    public class BackgroundPainter implements Painter<JComponent> {
    	public BackgroundPainter() {
    		super();
    	}
	    @Override
	    public void paint(Graphics2D g, JComponent comp, int width, int height) {
	        g.setColor(NotebookSetting.getTextBackground());
	        g.fillRect(0, 0, width, height);
	    }
    }

    private class AccessibleTabComponent extends AccessibleJScrollPane {
        //Inherit everything, override nothing.
    }

}
