package cologne.eck.tools;



/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;

/*
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel; */

//import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
//import cologne.eck.all_peas.gui.PeaDialog;
//import cologne.eck.all_peas.gui.PswDialogView;


public class FileTools {
	
	// used as buffer for large files
	private static final int bufferSize = 16 * 1024 * 1024; // 16 MB
	
	/**
	 * Get all files of a directory including sub-directories
	 * as an unsorted list. 
	 * 
	 * @param directory			the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an unsorted list of all included files 
	 * 			or null if the directory is null 
	 * 			or is not to a directory 
	 */
	public static ArrayList <File> listAllFilesOfFolder(File directory, boolean filesOnly) {
		
		if (directory == null) {
			System.err.println("FileTools: directory is null");
			return null;
		}
		if (! directory.isDirectory()) {
			System.err.println("FileTools: argument is not directory");
			return null;		
		}
		ArrayList<File> resultList = new ArrayList<File>();

        File[] fileList = directory.listFiles();        
        if (fileList == null) {
			System.err.println("FileTools: No files in directory");
			return null;		
        }
        int len = fileList.length;
        for (int i = 0; i < len; i++) {
        	File file = fileList[i];
        	//resultList.add(file);
            if (file.isFile()) {
            	resultList.add(file);
                //System.out.println(file.getAbsolutePath());
            } else if (file.isDirectory()) {
            	if (filesOnly == false){
            		resultList.add(file);
            	}
            	if (listAllFilesOfFolder(file, filesOnly) != null){
            		resultList.addAll(listAllFilesOfFolder(file, filesOnly));
            	}
            }
        }
        return resultList;
	}
	
	/**
	 * Get all names of files of a directory including sub-directories
	 * as an unsorted list. 
	 * 
	 * @param directoryName		the name of the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an unsorted list of all included file names 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	private static ArrayList <String> listAllFileNamesOfFolder(String directoryName, boolean filesOnly) {
		
		if (directoryName == null) {
			System.err.println("FileTools: directory nam is null");
			return null;
		}
		File directory = new File(directoryName);
		if (! directory.isDirectory()) {
			System.err.println("FileTools: argument is not directory");
			return null;		
		}
		ArrayList<String> resultList = new ArrayList<String>();

		File[] fileList = directory.listFiles();     
		if (fileList == null) {
			new UnexpectedValueException("fileList", "File[]", "is null").printDescription();
			return null;
		}
		int len = fileList.length;
		for (int i = 0; i < len; i++) {
			File file = fileList[i];
        	//resultList.add(file.getAbsolutePath());
            if (file.isFile()) {
            	resultList.add(file.getAbsolutePath());
                //System.out.println(file.getAbsolutePath());
            } else if (file.isDirectory()) {
            	if (filesOnly == false){
            		resultList.add(file.getAbsolutePath());
            	}
                resultList.addAll(listAllFileNamesOfFolder(file.getAbsolutePath(), filesOnly));
            }
        }
        return resultList;
	}
	/**
	 * Get all names of files of a directory including sub-directories
	 * as an array of Strings. 
	 * 
	 * @param directoryName		the name of the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an array of all included file names 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	public static String[] getAllFileNamesOfFolder(String directoryName, boolean filesOnly) {
		ArrayList <String> list = listAllFileNamesOfFolder(directoryName, filesOnly);
		if (list == null) {
			return null;
		} else {
			String[] array = new String[list.size()];
			return list.toArray(array);
		}
	}
	/**
	 * Get all files of a directory including sub-directories
	 * as an array of Strings. 
	 * 
	 * @param directory			the directory
	 * @param filesOnly			true if only file should be included
	 * 							false if files and directories should be included
	 * 
	 * @return	an array of all included files 
	 * 			or null if the directory name is null 
	 * 			or refers not to a directory 
	 */
	public static File[] getAllFilesOfFolder(File directory, boolean filesOnly) {
		ArrayList <File> list = listAllFilesOfFolder(directory, filesOnly);
		if (list == null) {
			return null;
		} else {
			File[] array = new File[list.size()];
//System.out.println("list size: " + array.length);
			return list.toArray(array);
		}
	}
	/**
	 * Get the file size in bytes
	 * 
	 * @param file	the  file
	 * @return			the number of bytes (long value)
	 */
	public final static long getFileSize(File file) {
		RandomAccessFile raf = null;
		if (file.isDirectory() ) {
			return 0;
		}
		long result = 0;
   		try {            			
				raf = new RandomAccessFile(file, "r");	            		
        		result = raf.length();
        		raf.close(); 
			} catch (FileNotFoundException e) { 
				// this will be thrown if file is not file and not directory
				if ( PeaProperties.isTestMode()) {
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + file.getAbsolutePath());
					System.err.println(new ExceptionHelper(e).getInfos());
				}
			} catch (IOException e) {
				if ( PeaProperties.isTestMode()) {			
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + file.getAbsolutePath());
					System.err.println(new ExceptionHelper(e).getInfos());
				}
			} catch (Exception e) {
				if ( PeaProperties.isTestMode()) {
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + file.getAbsolutePath());
					System.err.println(new ExceptionHelper(e).getInfos());
				}						
			}
   		return result;		
	}
	/**
	 * Get the file size in bytes
	 * 
	 * @param fileName	the name of the file
	 * @return			the number of bytes (long value)
	 */
	public final static long getFileSize(String fileName) {
		RandomAccessFile raf = null;
		File file = new File(fileName);
		if (file.isDirectory() ) {
			return 0;
		}
		long result = 0;
   		try {            			
				raf = new RandomAccessFile(file, "r");	            		
        		result = raf.length();
        		raf.close(); 
			} catch (FileNotFoundException e) { 
				// this will be thrown if file is not file and not directory
				if ( PeaProperties.isTestMode()) {
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + fileName);
					System.err.println(new ExceptionHelper(e).getInfos());
				}
			} catch (IOException e) {
				if ( PeaProperties.isTestMode()) {			
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + fileName);
					System.err.println(new ExceptionHelper(e).getInfos());
				}
			} catch (Exception e) {
				if ( PeaProperties.isTestMode()) {
					System.err.println(e.toString() + ", " + e.getMessage() + ": " + fileName);	
					System.err.println(new ExceptionHelper(e).getInfos());
				}						
			}
   		return result;		
	}
	/**
	 *  Get the number and overall size of files.
	 *  There is no warning for long execution time!
	 * 
	 * @param fileNames		an array of file names
	 * @return					an array of long values: the first value [0] is the
	 * 							number of the containing files (no directories), 
	 * 							the second [1] is the size of all included files. 
	 * 							There is no warning for long execution time!
	 */
	public static final long[] getNumberAndSize(String[] fileNames){
		
		long[] result = new long[2];
		
		if (fileNames == null){
			System.err.println("FileTools getNumberAndSize[]: fileNames null");
			return null;
		}
		
		int len = fileNames.length;
		if (len == 0) {
			result[0] = 0;
			result[1] = 0;
			return result;
		}
		
		RandomAccessFile raf = null;
		
		for (int i = 0; i < len; i++) {
			
			if (! new File(fileNames[i]).isFile()){
				// do not count directories and other
				continue;
			}
			
    		try {            			
				raf = new RandomAccessFile(fileNames[i], "r");	            		
        		result[1] += raf.length();
        		raf.close(); 
			} catch (FileNotFoundException e) {
				System.err.println(e.toString() +" FileTools getNumberAndSize[]: " +  e.getMessage() + ", " + fileNames[i]);
				System.err.println(new ExceptionHelper(e).getInfos());
			} catch (IOException e) {
				//read_write_failed = true;
				System.err.println(e.toString() + " FileTools getNumberAndSize[]: " +  e.getMessage() + ", " + fileNames[i]);
				System.err.println(new ExceptionHelper(e).getInfos());
				//e.printStackTrace();
			} catch (Exception e) {
				System.err.println(e.toString() + " - " + e.getMessage() + ", " + fileNames[i]);
				System.err.println(new ExceptionHelper(e).getInfos());
				//e.printStackTrace();								
			}
    		// only count files, not folders:
    		result[0] ++;
		}
		return result;
	}
	/**
	 *  Get the number of files of a directory - inclusive 
	 *  all files of sub-directories, but no directory itself.
	 *  There is no warning for long execution time!
	 * 
	 * @param directoryName		the name of the directory to check
	 * @return					an array of long values: the first value [0] is the
	 * 							number of the containing files (no directories), 
	 * 							the second [1] is the size of all included files. 
	 * 							There is no warning for long execution time!
	 */
	public static final long[] getNumberAndSize(String directoryName) {
		
		File directory = new File(directoryName);
		RandomAccessFile raf = null;

		long[] result = new long[2];

        File[] children = directory.listFiles();        
        if (children != null) {        	

            for (int i = 0; i < children.length; i++) {
            	if (children[i].isDirectory() ) {
            		long[] dirResult = getNumberAndSize(children[i].getAbsolutePath());
            		// function was broken because of extremeSizeWarning
            		// this avoids NullPointerException
            		if (dirResult == null) {
            			return null;
            		}
            		result[0] += dirResult[0];
            		result[1] += dirResult[1];    				
 
            	} else if (children[i].isFile() ) {

            		try {            			
						raf = new RandomAccessFile(children[i], "r");	            		
	            		result[1] += raf.length();
	            		raf.close(); 
					} catch (FileNotFoundException e) {
						//file_not_found = true;
						System.err.println("FileTools getNumberAndSize: " +  e.getMessage() + ", " + children[i]);
						System.err.println(new ExceptionHelper(e).getInfos());
						//e.printStackTrace();
					} catch (IOException e) {
						//read_write_failed = true;
						System.err.println("FileTools getNumberAndSize: " +  e.getMessage() + ", " + children[i]);
						System.err.println(new ExceptionHelper(e).getInfos());
						//e.printStackTrace();
					} catch (Exception e) {
						System.err.println(e.toString() + " - " + e.getMessage() + ", " + children[i]);
						System.err.println(new ExceptionHelper(e).getInfos());
						//e.printStackTrace();								
					}
            		// only count files, not folders:
            		result[0] ++;
            	}
            }
        }       
        return result;
    } 
	
	/**
	 * Checks a file if it is a valid to encrypt/decrypt or not. 
	 * Checks existence, read and write access, if empty or too large.
	 * If the file is invalid, the function return one of these five
	 * error string: 
	 * can_not_find, can_not_read, can_not_write, empty_file, file_too_large.
	 * If the file is valid, the functions returns null.
	 * 
	 * @param file			the file to check
	 * @param printErrors	print error messages to stderr (true) or not (false)
	 * 
	 * @return			an error string if the file is invalid
	 * 					or null if the file is valid
	 */
/*	public final static String checkAccess(File file, boolean printErrors) {

 		if ( ! file.exists() ) {	
 			if (printErrors == true) {
 				System.err.println("File does not exist: " + file.getAbsolutePath());
 			}
			return "can_not_find";
		}	
		if ( ! file.canRead() ) {
 			if (printErrors == true) {
 				System.err.println("No read access to file: " + file.getAbsolutePath());
 			}
			return "can_not_read";
		}
		if ( ! file.canWrite() ) {
 			if (printErrors == true) {		
 				System.err.println("No write access to file: " + file.getAbsolutePath());	
 			}
 			return "can_not_write";
		}						
		if ( file.length() == 0 && ! file.isDirectory() ) {
 			if (printErrors == true) {		
 				System.err.println("Empty file: " + file.getAbsolutePath());
 			}
			return "empty_file";
		}		
		if ( file.length() > Long.MAX_VALUE || file.length() < 0) {
 			if (printErrors == true) {		
 				System.err.println("Invalid file size: " + file.getAbsolutePath());
 			}
			return "file_too_large";
		}
		return null;
	}*/
	/**
	 * Overwrite file content and delete file.
	 * 
	 * @param fileName		the file name of the file to delete
	 * @param secureWipe	fill content with random data (expensive)
	 * 						after filling with 0x00 and 0xFF
	 * @return		an error message if an error occurred, otherwise null
	 */
	public final static String wipeFile(String fileName, boolean secureWipe) {
		
		String errorMessage = null;
		
		if (fileName == null) {
			new UnexpectedValueException("fileName", "String", "is null").printDescription();
			return "no file name";
		}

		File file = new File(fileName);

		if (file.isDirectory()) {
			// delete only empty directories: 
			File[] list = file.listFiles();
			if (list == null || list.length == 0) {
				file.delete();
			}
			return "is folder";
		}
		if (file.exists() && file.canRead() && file.canWrite()){				

			RandomAccessFile raf = null;
			long fileLength = 0;
			try {
				raf = new RandomAccessFile(file, "rw");
				fileLength = raf.length();
			} catch (FileNotFoundException e1) {
				System.err.println("Error " + fileName + "\n"
						+ e1.toString() + ", " + e1.getMessage());
				e1.printStackTrace();
				return errorMessage = "Error " + fileName + "\n"
						+ e1.toString() + ", " + e1.getMessage();
			} catch (IOException e) {
				System.err.println("Error " + fileName + "\n"
						+ e.toString() + ", " + e.getMessage());
				e.printStackTrace();
				return errorMessage = "Error " + fileName + "\n"
						+ e.toString() + ", " + e.getMessage();				
			}
			// number of buffer blocks
			long blockNumber = fileLength / bufferSize;
			// buffer containing bytes to overwrite the content
			byte[] buffer = null;
			// the rest of the file after buffer blocks
			byte[] restBuffer = new byte[(int) (fileLength - (blockNumber * bufferSize))];
			int restLength = restBuffer.length;
			if ( blockNumber > 0 ) { // file > 16 MB
				buffer = new byte[bufferSize];// initialized with 0
			}
			try {
				// Overwrite with zero
				for (long blockCounter = 0; blockCounter < blockNumber; blockCounter++) {	    		 
					raf.seek(blockCounter * bufferSize);
					raf.write(buffer);
				}
				if (restLength > 0) {
					raf.seek( fileLength - restLength);
					raf.write(restBuffer);	    		 
				}

				// Overwrite with 1
				if (buffer != null) {
					Arrays.fill(buffer, (byte) 0xFF);
				}
				Arrays.fill(restBuffer, (byte) 0xFF);
				for (long blockCounter = 0; blockCounter < blockNumber; blockCounter++) {	    		 
					raf.seek(blockCounter * bufferSize);
					raf.write(buffer);
				}
				if (restLength > 0) {
					raf.seek( fileLength - restLength);
					raf.write(restBuffer);	    		 
				}	    	 

				if (secureWipe == true) {
					/// Overwrite with random byte
					SecureRandom random = new SecureRandom();
					// fill file with random data
					for (long blockCounter = 0; blockCounter < blockNumber; blockCounter++) {	
						// fill buffer with random bytes
						random.nextBytes(buffer);	    		 
						raf.seek(blockCounter * bufferSize);
						raf.write(buffer);
					}
					if (restLength > 0) {
						// fill buffer with random bytes
						random.nextBytes(restBuffer);
						raf.seek( fileLength - restLength);
						raf.write(restBuffer);	    		 
					}
				} 
				raf.close();
			} catch (IOException ioe) {
				System.err.println("No access to file " + fileName);
				return errorMessage = "Deletion failed. No access to file " + fileName;
			} catch (Exception e) {
				System.err.println("Unexpected error " + fileName + "\n"
						+ e.toString() + ", " + e.getMessage());
				return errorMessage = "Unexpected error " + fileName + "\n"
						+ e.toString() + ", " + e.getMessage();
			}
			file.delete();	    	 
	    	 
	     } else {
	    	 System.err.println("No access to file " + fileName);
	    	 return errorMessage = "Deletion failed. No access to file " + fileName;
	     }
	    return errorMessage;		
	}
	/**
	 * Copy a file via FileChannel and create the destination
	 * file if necessary
	 * 
	 * @param sourceFile		the file to copy
	 * @param destinationFile	the new file
	 * 
	 * @return	a localized error message if copy failed
	 * 			or null for success
	 */
	public static String copyFiles(File sourceFile, File destinationFile) {
		

		FileChannel sourceChannel = null;
		FileChannel destChannel = null;
		FileInputStream fileInput = null; 
		FileOutputStream fileOutput = null;
		try {
			if (destinationFile.exists() == false) {
				destinationFile.getParentFile().mkdirs(); 
				destinationFile.createNewFile();
			}
			fileInput = new FileInputStream(sourceFile);
			sourceChannel = fileInput.getChannel();
			fileOutput = new FileOutputStream(destinationFile);
			destChannel = fileOutput.getChannel();
			destChannel.transferFrom(sourceChannel, 0, sourceChannel.size());
		} catch (FileNotFoundException e) {
			System.err.println(new ExceptionHelper(e).getInfos());	
			return sourceFile.getAbsolutePath() + " -> " 
					+ destinationFile.getAbsolutePath() + ":\n " 
			+ e.getLocalizedMessage();
		} catch (IOException e) {
			System.err.println(new ExceptionHelper(e).getInfos());	
			return sourceFile.getAbsolutePath() + " -> " 
					+ destinationFile.getAbsolutePath() + ":\n " 
			+ e.getLocalizedMessage();
		} finally{
			if (fileInput != null) {
				try {
					fileInput.close();
				} catch (IOException e) {
					System.err.println("Closing stream failed: "+ sourceFile.getAbsolutePath());
					System.err.println(new ExceptionHelper(e).getInfos());	
				}
			}
			if (sourceChannel != null) {
				try {
					sourceChannel.close();
				} catch (IOException e) {
					System.err.println("Closing channel failed... ");
					System.err.println(new ExceptionHelper(e).getInfos());	
				}
			}
			if (fileOutput != null) {
				try {
					fileOutput.close();
				} catch (IOException e) {
					System.err.println("Closing stream failed: "+ destinationFile.getAbsolutePath());
					System.err.println(new ExceptionHelper(e).getInfos());	
				}
			}
			if (destChannel != null){
				try {
					destChannel.close();
				} catch (IOException e) {
					System.err.println("Closing channel failed... ");
					System.err.println(new ExceptionHelper(e).getInfos());	
				}
			}
		}
		return null;
	}
	/**
	 * Create an empty new file with all required directories. 
	 * This will overwrite an existing file without confirmation 
	 * (confirmation must be done before by askToOverwriteExistingFile)
	 * 
	 * @param fileNameWithPath	the name of the file to create
	 * 
	 * @return	the file or null 
	 */
	public static File createEmptyFile(String fileNameWithPath) {
		File file = new File(fileNameWithPath);
		if (! file.exists()) {
			// create required directories
			file.getParentFile().mkdirs();
			// create file
			try {
				file.createNewFile();
			} catch (IOException e) {
				System.err.println("Creating file failed: "+ fileNameWithPath);
				System.err.println(new ExceptionHelper(e).getInfos());
			}
		} 
		return file;
	}
	

	
    /**
     * Check if files already exists and ask to overwrite
     * 
     * @param fileNames	the file names to check
     * 
     * @return	the file names to write (overwriting confirmed or
     * 			not yet exist)
     */
	public static String[] askToOverwriteFiles(String[] fileNames) {
		
		return PeaProperties.getVmBridge().askToOverwriteFiles(fileNames);
/*		*/
	}
	
    /**
     * Check if a file already exists and ask to overwrite
     * 
     * @param file	the file to check
     * 
     * @return	true, if the file should be overwritten or
     * 			if the file does not exist, 
     * 			false if the file should not be written
     */
	public static boolean askToOverwriteFile(File file) {
		return PeaProperties.getVmBridge().askToOverwriteFile(file);
		
/*		*/
	}
	
	/**
	 * Checks if the content of two file is equal
	 * by file size and optionally by content. 
	 * If files are encrypted, the content is never equal
	 * because of different/unique nonce. 
	 * 
	 * @param fileName1	name of file 1 to check
	 * @param fileName2	name of file 2 to check
	 * @param encrypted	true if the file are encrypted
	 * 
	 * @return	0 if content is equal, 1 if different, -1 comparison if failed
	 */
/*	public static int checkEqualContent(String fileName1, String fileName2, boolean encrypted) {
		
		if (fileName1.equals(fileName2)) {
			System.err.println("FileTools: Comparison of equal files: " + fileName1);
			return 0;
		}
		File file1 = new File(fileName1);
		File file2 = new File(fileName2);
		if ( ! file1.exists() || ! file2.exists()) {
			new UnexpectedValueException("File", "file1 or file2", "do not exist").printDescription();
			return -1;
		}
		if ( ! file1.isFile() || ! file2.isFile()) {
			new UnexpectedValueException("File", "file1 or file2", "are not files").printDescription();
			return -1;
		}
		
		long file1Len = file1.length();
		if (file1Len != file2.length()) {
System.out.println("File length differs: " + fileName1 + " and " + fileName2);
			return 1;// not equal
		}		
		if (encrypted == true) {
			// can't check the content because of different nonce
			return 0; // probably equal
		}
		
		// for small file size: readAllBytes, for large: buffer
		int bufferSize = 8192;
		if (file1Len <= bufferSize) {

			try {
				byte[] f1 = Files.readAllBytes(file1.toPath());
				byte[] f2 = Files.readAllBytes(file2.toPath());
				if (Arrays.equals(f1, f2)){
					return 0;
				} else {
//Help.printBytes("f1", f1);
//Help.printBytes("f2", f2);
					return 1;
				}
			} catch (IOException e) {
				System.err.println(new ExceptionHelper(e).getInfos());
				return -1;
			}

		} else { // large files > 8192 bytes
			
			// TODO test
			RandomAccessFile raf1 = null;
			RandomAccessFile raf2 = null;
			
			try {
				raf1 = new RandomAccessFile(file1, "r");
				raf2 = new RandomAccessFile(file2, "r");

				byte[] dest1      = new byte[bufferSize];
				byte[] dest2      = new byte[bufferSize];
				int    offset    = 0;
				
				while (offset + bufferSize <= file1Len) {
					raf1.seek(offset);
					raf2.seek(offset);
					//int bytesRead1 = 
					raf1.read(dest1);
					//int bytesRead1 = raf1.read(dest1, offset, bufferSize);
					//int bytesRead2 = 
					raf2.read(dest2);
					if (Arrays.equals(dest1, dest2)){
						// 
					} else {
						raf1.close();
						raf2.close();
						return 1;						
					}					
					offset += bufferSize;
				}
				raf1.close();
				raf2.close();
				return 0;
			} catch (FileNotFoundException e) {
				System.err.println(new ExceptionHelper(e).getInfos());
				return -1;
			} catch (IOException e) {
				System.err.println(new ExceptionHelper(e).getInfos());
				return -1;
			}
		}		
	}*/
	
	
	/**
	 * Get all parent directory names of a file as list. 
	 * The direct directory is the first element of the
	 * list, the root directory the last element. 
	 * 
	 * @param file	the file 
	 * 
	 * @return		the list of all parent directory names, where
	 * 				the direct parent directory is the first element
	 */
/*	public static ArrayList<String> listParentDirectoryNames(File file) {
		!!! untested !!!
		ArrayList<String> list = new ArrayList<String>();
		
		String parentName = file.getParent();
		if (parentName == null) {
			return null;
		}
		File parent = new File(parentName);		
		while (parent.isDirectory()) {
			list.add(parentName);
			parentName = parent.getParent();
			if (parentName == null ){
				break;
			}
			parent = new File(parentName);
			if (parent.exists() == false) {
				break;
			}
		}
		if (list.isEmpty()){
			return null;
		} else {
			return list;
		}
	} */
}
